	.z280
;
;  CSTART.ASM	- Control Cross-C all-purpose startup code.
;
;  Copyright 1991 Softools, Inc.  All rights reserved.
;  Simple Version for ELZET 80 ECB-CPUZ280 with CP/M3
;
;  Since this file is self-configuring using several assemble-time
;  prompts, it should be assembled separately for each application
;  program being developed.
;
;  If a pre-startup module is used to initialize other hardware,
;  interrupt vectors, etc., the global entry point CSTART should
;  be jumped to from that module.  Otherwise, this module is placed
;  at location 0 and will handle the starup.  The state of interrupts
;  is NOT changed by this module.  Therefore, if a pre-startup module
;  turns them off, main () or this module should turn them back on.
;  The SP and a banked program MMU is initialized by this module.
;
;  The following memory configuration is used in the Control Cross-C
;  environment.  Note that not all segments are necessarily used in
;  all applications.
;
;  Segment	Contains
;  ==================================================================
;  CODE 	Startup code, libraries and non-banked compiled code.
;  CONST	String literals, const objects and
;			initializers for automatic aggregates.
;  DATA 	Initialized data area.	May be copied from ROM to RAM.
;  BSS		Uninitialized data area.  This may be zeroed on startup.
;  BSSZ 	Initialized by 0 data.	This may be zeroed on startup.
;  STACK	The amount reserved for the CPU stack
;		The remmaining area is left over and used for the dynamic heap.
;
;  --------------------------------------------------------------------------
;  A non-banked program is linked with SLINK using the following commands:
;    (This is only an example.	The addresses used need to be changed
;    to reflect the target system and application program.)
;
;  (CODE, start=0)		; Place CODE at 0
;  (CONST)			; Place constants next
;  (DATA)			; Place DATA next in ROM
;  program modules and libraries should be listed here
;  (XDATA<DATA,start=8000)	; Reserve space for copied initialized DATA
;  (BSS)			; BSS is after DATA (at 8000)
;  (BSSZ)			; BSSZ is after BSS
;  (STACK)			; The CPU stack follows data
;
;  --------------------------------------------------------------------------
;  A banked program is linked with SLINK using the following commands:
;    (This is only an example.	The addresses used need to be changed
;    to reflect the target system and application program.)
;
;  (CODE, start=0)		; Place CODE at 0
;  (BANK_CODE)			; Place BANK_CODE next
;  (CONST)			; Place constants next
;  (DATA)			; Place DATA next in ROM, locate at
;  non-banked program modules and libraries should be listed here
;  (BCODE=CODE, banked) 	; Alias remaining CODE segments to BCODE
;  banked program modules and libraries should be listed here
;  (XDATA<DATA,start=C000)	; Reserve space for copied initialized DATA
;  (BSS)                        ; BSS is after DATA (at C000)
;  (BSSZ)			; BSSZ is after BSS
;  (STACK)                      ; The CPU stack follows data
;
;  --------------------------------------------------------------------------
;  The following assembly-time questions are asked to configure the
;    application:
;
;  We prompt for the CPU stack size.  This eliminates editing
;    this file just to change the stack size.
;
;  We prompt to find out if the system is battery backed.
;    If it is, then we do not zero-fill BSS and
;    We prompt if the system needs a power-up test for initialization.
;      If it does, then a variable stored in segment DATA is compared
;      to a known default value.  If the variable contains this value,
;      the initialization is skipped.  Otherwise, it is performed.
;    If not, segment BSS is never zero filled.
;
Y	equ	1
N	equ	0
;
size::	.input	"Application CPU stack size? "
bb_sys: .input	"Is application running in a battery-backed system (Y/N)? "
	.if	bb_sys
bb_test:.input	"Do data initialization if a battery-backed test fails (Y/N) ? "
	.else
bb_test equ	N
	.endif
;
__stack_size::	.equ	even size
;
        .cseg                           ; Define implicit segment ordering
	.dseg
	.bseg
	.defseg BSSZ
	.defseg XDATA,data
	.defseg STACK,stack,align=2	; Define STACK segment
	.seg	STACK
	.ds	__stack_size
__stack::.equ	$			; Initial SP
;
	.cseg
;
cstart::ld	sp, __stack
;
;  If checking a battery backed system, this code defines in segment
;  DATA a test pattern and checks to see if it valid.  If it is not,
;  then BSS, BSSZ, and DATA are initialized.  If it matches, this
;  bypasses all initialization and continues below.
;
	.if	bb_test
	.dseg
bb_patt:db	0C3h,0F0h,0AAh,03Ch,0F0h,055h
	.cseg
	ld	hl,bb_patt		; Test pattern
	ld	b,6			; 6-byte pattern
	ld	de,0
..test: ld	a,d
	add	(hl)
	ld	d,a
	ld	a,e
	xor	(hl)
	ld	e,a
	inc	hl
	djnz	..test			; All bytes XORed is 0
	ld	a,low (0C3h+0F0h+0AAh+03Ch+0F0h+055h)
	sub	d			; Check sum of data
	jr	nz,..fail
	or	e			; Data XORed is 0
	jr	z,..skip		; Skip initialization
..fail: .endif
;
;  Zero out the BSS data area.	If the size of BSS is not zero, this
;  zeros BSS PLUS ONE BYTE (due to a quicker LDIR fill method used).
;  This is NOT done if the system is battery-backed!
;
	.if	! bb_sys || bb_test
	ld	bc, sgsz BSS		; Size of BSS
	ld	a,b
	or	c
	jr	z,..bssz
	ld	hl, sgst BSS		; Logical start of BSS
        ld      (hl),0                  ; 0 first byte
        ld      d,h
	ld	e,l
	inc	de			; de = hl + 1
	ldir
	.endif
..skip:
;
;  Zero out the BSSZ data area.  This is for data with initializers of
;  0 (e.g. int x = 0;) and MUST be done even in a battery backed system.
;  (Unless the battery-backed test passes and all initialization is
;  skipped.
;
..bssz: ld	hl, sgst BSSZ		; Start of BSSZ
	ld	bc, sgsz BSSZ		; Size of BSSZ
..loop: ld	a,b
	or	c
	jr	z,..data
	ld	(hl),0			; Zero BSSZ
	inc	hl
	dec	bc
	jr	..loop
;
;  Copy initialized data to segment DATA.
;
..data: ld	bc, sgsz DATA		; Size of DATA
	ld	a,b
	or	c
	jr	z,..main		; No data
	ld	de, sgst DATA		; Logical start of DATA now mapped in.
	ld	hl, sgat DATA		; Physical start of DATA in ROM.
	ldir
;
;  Call main ()
;
	push	ix
	call	__initio##
	pop	ix
..main: call	_main##
_abort::
_exit::
;
	jp	0

;
;  Define variables defining free RAM available for the heap.
;  If the heap is moved, __heap_end must remain >= __heap_start.
;  If malloc/free are not used, then it doesn't matter.  These
;  must address memory that is always mapped in.  It's probable
;  but very important that the heap start is NOT 0.
;
__heap_start::
	dw	__stack 		; Stack top defines start of heap
__heap_end::
	dw	0eFFFh			; The end is at 0eFFF
;
	.end	cstart
