/*----------------------------------------------------------------------
 *			 TMS340 Graphics Library
 *          Copyright (c) 1990 Texas Instruments Incorporated.
 *                         All Rights Reserved
 *----------------------------------------------------------------------
 * Demonstration Program
 *
 *   Draw text to the screen using the same typeface in a variety of
 *   font sizes.  Lincoln's Gettysburg Address is printed to the screen
 *   using SYS16 font , SYS24 font and Corpus Christi font sizes 15, 16,
 *   26, 29, and 49.  Each frame shows a different font size.
 *   A delay is inserted between successive frames.
 *----------------------------------------------------------------------
 *   03/27/90...Original version written..................J.R. Van Aken
 *   09/25/90...Cut no. of fonts from 12 to 10............J.R. Van Aken
 *   04.02.1993	Anpassung an GDP340 GM			Reinhard Niebur
 *----------------------------------------------------------------------
 */

#include "gmtest.h"

CONFIG cfg;
FONTINFO fontinfo;
short hres,vres,x,y;
short patn,penw,penh;
char s[129];

#define N_LINES	25	/* number of lines of text */
#define NFONTS	7	/* number of fonts */

static short index[NFONTS];

static char *str[N_LINES] =
{
    "Four score and seven years ago our fathers brought forth, upon this",
    "continent, a new nation, conceived in Liberty, and dedicated to the",
    "proposition that all men are created equal.",
    "",
    "Now we are engaged in a great civil war, testing whether that nation,",
    "or any nation so conceived, and so dedicated, can long endure.  We",
    "are met here on a great battlefield of that war.  We have come to",
    "dedicate a portion of it as a final resting place for those who here gave",
    "their lives that the nation might live.  It is altogether fitting and proper",
    "that we should do this.",
    "",
    "But in a larger sense we can not dedicate, we can not consecrate,",
    "we can not hallow this land.  The brave men, living and dead, who",
    "struggled here, have consecrated it far above our poor power to",
    "add or detract.  The world will little note, nor long remember, what",
    "we say here, but can never forget what they did here.  It is for us,",
    "the living, rather to be dedicated here to the unfinished work",
    "which they have, thus far, nobly carried on.  It is rather for us to",
    "be here dedicated to the great task remaining before us, that from",
    "these honored dead we take increased devotion to that cause for",
    "which they gave the last full measure of devotion, that we here",
    "highly resolve that these dead shall not have died in vain; that this",
    "nation shall have a new birth of freedom; and that this government",
    "of the people, by the people, for the people, shall not perish from",
    "the earth."
};

/* Print error message and exit. */
void error(char *str)
{
    printf("[ Error: ");
    if (!*str)
        printf("Unexpected EOF");
    else
        printf("%s",str);
    printf(" ]\n");
    exit(1);
}

void main(void)
{
short nfonts, i, j, k, h;

    if (!gm_init())
        error("No Response from GDP");

    /* Initialize graphics environment. */
    if (!set_config(MODE, 1))
	error("Invalid graphics mode number");
    get_config(&cfg);
    hres = cfg.mode.disp_hres;	 /* horizontal resolution */
    vres = cfg.mode.disp_vres;	 /* vertical resolution */

     /* Deinstall all fonts to get place for new fonts */
    init_text();

    /* Install as many fonts as will fit in the font table. */
    for (nfonts = 0;
	   nfonts < NFONTS && (index[nfonts] = install_hard_font(nfonts));
	     nfonts++)
	;

    /* Each loop below draws text in one of the fonts. */
    for (;;)
	for (i = 0; i < nfonts; i++)
	{
	    select_font(index[i]);
	    get_fontinfo(index[i], &fontinfo);
	    h = fontinfo.charhigh;
	    x = y = 0;
	    clear_page(BLACK);
	    for (j = 0; j < N_LINES && y < vres; j++, y += h)
		text_out(x, y, str[j]);
/*	    delay(75); */	   /* 1.25-second delay */
  for(k=0; k<75; k++)
    wait_scan(vres);
	    if (kbhit())
	    {
		getch();
		clear_page(BLACK);
		return;
	    }
	}
}

