/*----------------------------------------------------------------------
 *			 TMS340 Graphics Library
 *       Copyright (c) 1987-1990 Texas Instruments Incorporated.
 *                         All Rights Reserved
 *----------------------------------------------------------------------
 * Demonstration Program
 *
 *   Test the polygon fill function.  Draw a series of filled polygons
 *   to the screen using the fill_polygon function.  The polygons are
 *   defined by five moving points which move in a straight-line
 *   direction until they bounce off the sides of the screen.
 *----------------------------------------------------------------------
 * Revision History:
 *   03/31/87...Original version written...................J.R. Van Aken
 *   06/02/87...Removed bounding polylines from polygons...J.R. Van Aken
 *   03/27/90...Modified for general portability...........J.R. Van Aken
 *   04.02.1993	Anpassung an GDP340 GM			Reinhard Niebur
 *----------------------------------------------------------------------
 */

#include "gmtest.h"

CONFIG cfg;
FONTINFO fontinfo;
short hres,vres;
short patn,penw,penh;
char s[129];

#define NVERTS	6	    /* number of polygon vertices	*/

/* Print error message and exit. */
void error(char *str)
{
    printf("[ Error: ");
    if (!*str)
        printf("Unexpected EOF");
    else
        printf("%s",str);
    printf(" ]\n");
    exit(1);
}

void main(void)
{
static POINT2D xy[NVERTS], vxy[NVERTS];
short i, k;
long color0, color1, x, y;

    if (!gm_init())
        error("No Response from GDP");

    /* Initialize graphics environment. */
    if (!set_config(MODE, 1))
	error("Invalid graphics mode number");
    clear_screen(0);
    get_config(&cfg);
    hres = cfg.mode.disp_hres;	 /* horizontal resolution */
    vres = cfg.mode.disp_vres;	 /* vertical resolution */

    /* Adjust shape and movement parameters to screen dimensions. */
    x = (long)hres << 16;
    y = 0;
    for (i = 0; i < NVERTS; i++) {
	x -= y/4;
	y += x/4;
	xy[i].x = hres/2;
	xy[i].y = vres/2;
	vxy[i].x = (short)((3*x) >> 16+6);
	vxy[i].y = (short)((3*y) >> 16+6);
    }

    /* Loop below draws polygons to screen until kbhit. */
    k = 0;
    color0 = 0;
    color1 = -1;
    patn = 0;
    for (;;)
    {
	set_colors(color1, color0);
	set_hard_patn(patn);

	/* Alternate solid- and pattern-filled polygons. */
	if (++patn == NUM_PATNS)
	    patn = 0;
	if (k++ & 1)
	{
	    fill_polygon(NVERTS, xy);
	    color0++;
	    color1--;
	}
	else
	    patnfill_polygon(NVERTS, xy);

	/* Check for collisions with edges of screen. */
	for (i = 0; i < NVERTS; i++)
	{
	    xy[i].x += vxy[i].x;
	    if (cpw(xy[i].x, vres/2))
	    {
		vxy[i].x = -vxy[i].x;
	    }
	    xy[i].y += vxy[i].y;
	    if (cpw(hres/2, xy[i].y))
	    {
		xy[i].y -= vxy[i].y;
		vxy[i].y = -vxy[i].y;
	    }
	}
	if (kbhit())
	    break;
    }
    getch();
    clear_page(BLACK);
}

