/*----------------------------------------------------------------------*/
/*									*/
/*	Copyright (c) 1992	ELZET 80 Mikrocomputer			*/
/*				All Rights Reserved			*/
/*									*/
/*----------------------------------------------------------------------*/
/*   GDP340 - Graphics Manager						*/
/*----------------------------------------------------------------------*/
/*   Demonstration Program						*/
/*									*/
/*   Test cursor functions.						*/
/*									*/
/*   Notes:								*/
/*----------------------------------------------------------------------*/
/*   Revision history:							*/
/*   10.02.1993	Original Version Written		Reinhard Niebur	*/
/*----------------------------------------------------------------------*/

#include "gmtest.h"

CONFIG cfg;
FONTINFO fontinfo;
short hres,vres,x,y;
short patn,penw,penh;
char s[129];

unsigned short ArrowLData[] =
{
    0x0010, 0x0018, 0x001c, 0xfffe, 0xffff, 0xfffe, 0x001c, 0x0018, 0x0010,
    0x0000, 0x0000, 0x0008, 0x000c, 0x7ffe, 0x000c, 0x0008, 0x0000, 0x0000
};
CURSOR ArrowL =
{
    0x0000, 0x0004, 0x0010, 0x0009, 0x0010,
    0x0ffffffffL, 0x0020, 0x0020, 0x0ffffffffL, (PTR) 0
};

/* Print error message and exit. */
void error(char *str)
{
    printf("[ Error: ");
    if (!*str)
        printf("Unexpected EOF");
    else
        printf("%s",str);
    printf(" ]\n");
    exit(1);
}

void main(void)
{
short len, dx, dy, px, py;
ulong fgc = 0;
PTR pArrowL;

    if (!gm_init())
        error("No Response from GDP");

    /* Initialize graphics environment. */
    if (!set_config(MODE, 1))
	error("Invalid graphics mode number");
    get_config(&cfg);
    hres = cfg.mode.disp_hres;	 /* horizontal resolution */
    vres = cfg.mode.disp_vres;	 /* vertical resolution */
    get_fontinfo(-1, &fontinfo);

    len = (ArrowL.height * ArrowL.pitch << 1) >> 3;
    ArrowL.data = gsp_malloc((ulong)len);
    pArrowL = gsp_malloc(sizeof(ArrowL));
    host2gsp(&ArrowL, pArrowL, sizeof(ArrowL), 0);
    host2gsp(ArrowLData, ArrowL.data, len, 0);

    x = hres/2;
    y = vres/2;
    dx = dy = 2;
    clear_page(BLUE);
    set_fcolor(GREEN);
    fill_rect(hres/2, vres/2, 0, vres/2);
    set_fcolor(RED);
    fill_rect(hres/2, vres/2, hres/2, 0);
    set_fcolor(LIGHT_GRAY);
    fill_rect(hres/2, vres/2, hres/2, vres/2);

    set_draw_origin(hres/2-13*fontinfo.charwide, vres/2-5*fontinfo.charhigh);
    x = 0;
    y = 0;
    text_out(x, y, "   GRAPHICS CURSOR DEMO   ");
    y += fontinfo.charhigh;
    text_out(x, y, " '1' Arrow1    '2' Arrow2 ");
    y += fontinfo.charhigh;
    text_out(x, y, " '3' Arrow3    '4' Arrow4 ");
    y += fontinfo.charhigh;
    text_out(x, y, " '5' Arrow5    'B' Bucket ");
    y += fontinfo.charhigh;
    text_out(x, y, " 'C' Cross     'G' Glass  ");
    y += fontinfo.charhigh;
    text_out(x, y, " 'P' Pencil    't' Text16 ");
    y += fontinfo.charhigh;
    text_out(x, y, " 'T' Text24    'W' Watch  ");
    y += fontinfo.charhigh;
    text_out(x, y, " 'D' Downloaded cursor    ");
    y += fontinfo.charhigh;
    text_out(x, y, " ' ' DefaultCursor        ");
    y += fontinfo.charhigh;
    text_out(x, y, " ESC to abort             ");

    set_curs_xy(x,y);
    get_curs_xy(&px, &py);
    if((x!=px) || (y!=py))
	error("get_curs_xy()");
    set_curs_state(ON);
    if(get_curs_state() != 1)
	error("get_curs_state()");

    for(;;)
    {
	delay(1);
	x += dx;
	y += dy;
	if (x >= hres) { dx = -dx; x = hres;}
	if (x <= 0) { dx = -dx; x = 0; }
	if (y >= vres) { dy = -dy; y = vres; }
	if (y <= 0) { dy = -dy; y = 0; }
	set_curs_xy(x,y);
	if(kbhit())	/* change shape	*/
	{
	    if ((++fgc) == cfg.mode.palet_size)
		fgc = 1;
	    switch (getch())
	    {
		case ' ': set_curs_shape((PTR)DefaultCursor);
			  set_cursattr(fgc,0xffffffffl,8,1); break;
		case '1': set_hard_curs_shape(Arrow1);
			  set_cursattr(fgc,0xffffffffl,8,1); break;
		case '2': set_hard_curs_shape(Arrow2);
			  set_cursattr(fgc,0xffffffffl,8,1); break;
		case '3': set_hard_curs_shape(Arrow3);
			  set_cursattr(fgc,0xffffffffl,8,1); break;
		case '4': set_hard_curs_shape(Arrow4);
			  set_cursattr(fgc,0xffffffffl,8,1); break;
		case '5': set_hard_curs_shape(Arrow5);
			  set_cursattr(fgc,0xffffffffl,8,1); break;
		case 'b':
		case 'B': set_hard_curs_shape(Bucket);
			  set_cursattr(fgc,0xffffffffl,8,1); break;
		case 'c':
		case 'C': set_hard_curs_shape(Cross);
			  set_cursattr(fgc,0xffffffffl,8,1); break;
		case 'g':
		case 'G': set_hard_curs_shape(Glass);
			  set_cursattr(fgc,0xffffffffl,8,1); break;
		case 'p':
		case 'P': set_hard_curs_shape(Pencil);
			  set_cursattr(fgc,0xffffffffl,8,1); break;
		case 't': set_hard_curs_shape(Text16);
			  set_cursattr(fgc,0xffffffffl,10,1); break;
		case 'T': set_hard_curs_shape(Text24);
			  set_cursattr(fgc,0xffffffffl,10,1); break;
		case 'w':
		case 'W': set_hard_curs_shape(Watch);
			  set_cursattr(fgc,0xffffffffl,8,1); break;
		case 'd':
		case 'D': set_curs_shape(pArrowL);
			  set_cursattr(fgc,0xffffffffl,0x0020,0x0020); break;
		case 0x1b: set_curs_state(OFF);
			   clear_page(BLACK);
			   return;
	    }
	}
    }
}
