;--------------------------------------------------------------------------;
;									   ;
;   Copyright (c) 1992	ELZET 80 Mikrocomputer				   ;
;			All Rights Reserved				   ;
;									   ;
;--------------------------------------------------------------------------;
;   GDP340 - Graphics Manager						   ;
;--------------------------------------------------------------------------;
;   Function library host side						   ;
;									   ;
;   Handle GDP340 GM with Z80/180/280-CPU's using I/O mapped communication.;
;									   ;
;   Notes: 								   ;
;   Input parameter are passed over the stack using following conventions: ;
;	Arguments must be pushed in revers order of their declaration,	   ;
;	8 bit arguments are pushed as 16 bit with high byte set to zero,   ;
;	32 bit arguments are passed with high word pushed first,	   ;
;       After return, the amount of pushed arguments on the stack	   ;
;	must be removed by the caller.					   ;
;									   ;
;   Return parameter are passed in registers using following convention:   ;
;	 8 bit: A							   ;
;	16 bit: HL	high byte in H					   ;
;	32 bit: BC/DE	high word in BC					   ;
;									   ;
;   Register AF, BC, DE, HL are used and destroyed.			   ;
;									   ;
;   For Z280 CPU: I/O page must be set to ECB-Bus-Page			   ;
;--------------------------------------------------------------------------;
;   Revision history:							   ;
;   19.03.1993	Original Version Written, Version 2.0c	Reinhard Niebur	   ;
;   05.07.1993	New I/O Address Layout, Version 2.0c	Reinhard Niebur	   ;
;--------------------------------------------------------------------------;
;
Y	equ	1
N	equ	0
;
cpu::	.input	"Application CPU ? Z"
	.if	cpu=280
	.z280
	.else
	.z80
	.endif

;	.include "../include/gmtypes.inc"
	.include "../include/gmhctrl.inc"
	.include "../include/gmfunc.inc"

;	ECB Host port addresses
haddhh	equ	0D2h	; Address high word
haddhl	equ	0D3h
haddlh	equ	0D4h	; Address low word
haddll	equ	0D5h
hctrlh	equ	0D6h	; Host control register
hctrll	equ	0D7h
hdatah	equ	0D0h	; Host data register
hdatal	equ	0D1h

ParAddr	.macro	n
   .if	cpu=280
	lda	hl,(sp+n)
   .else
	ld	hl,n
	add	hl,sp
   .endif
	.endm

Par	.macro	n
   .if	cpu=280
	ld	hl,(sp+n)
   .else
	ld	hl,n
	add	hl,sp
	ld	a,(hl)
	inc	hl
	ld	h,(hl)
	ld	l,a
   .endif
	.endm

Rev_K	.macro
   .if	cpu=280
	in	a,(hctrll)	;**** see Z280 Rev. K attachment ***
   .endif
	.endm

	.cseg

	.align	2
; void gm_cmd(ushort cmd)
gm_cmd:
; If last command doesn't return any value
; Wait until GDP is ready
	call	wait_msgout0
	ld	a,(_cmdbuf)	; Set command buffer address
	out	(haddll),a
	ld	a,(_cmdbuf+1)
	out	(haddlh),a
	ld	a,(_cmdbuf+2)
	out	(haddhl),a
	ld	a,(_cmdbuf+3)
	out	(haddhh),a
	ld	a,l		; Put command
	out	(hdatal),a
	ld	a,h
	out	(hdatah),a
	Rev_K
	ret

	.align	2
; void gm_sync_cmd(short wait)
gm_sync_cmd:
	in	a,(hctrll)	; Send message 1
	and	~(INTOUT+MSGIN)	; Clear message out interrupt bit and msgin
	or	MSGIN1
	out	(hctrll),a
wait_msgout1:			; Wait until GDP send message 1
	in	a,(hctrll)
	and	MSGOUT
	cp	MSGOUT1
	jr	nz,wait_msgout1
	in	a,(hctrll)	; Send message 0
	and	~(INTOUT+MSGIN)	; Clear message out interrupt bit and msgin
;	or	MSGIN0		; MSGIN0 is 0, bits are still cleared
;
; Interrupt driven communication is possible on message base. If we uncomment
; next line, the GM GDP side part will answer with message 0 with INTOUT bit
; set. Hardware on GDP340 board will then produce an interrupt request on
; ECB-Bus. If Jumper J1 on GDP340 board is closed, a vector interrupt with
; fixed vector 0f0h on ECB-Bus connector pin 21c will be generated.
; If jumper J1 is open, an unvectored interrupt on pin 21a will be generated.
; 
;	or	INTIN		; Set message in interrupt bit
;
	out	(hctrll),a
	Rev_K
	ld	a,l		; If wait = 0, then command is a function that
	or	h		; doesn't return any value, so we must not...
	ret	z
; void wait_msgout0(void)
wait_msgout0:
	in	a,(hctrll)	; Wait until GDP send message 0
	and	MSGOUT
	cp	MSGOUT0
	jr	nz,wait_msgout0
	ret

	.align	2
; short gm_init(void)
_gm_init::
; Preinitialize GDP340 control register: Frameload, 25.175 MHz
	ld	a,CNTL_ADR_LL
	out	(haddll),a
	ld	a,CNTL_ADR_LH
	out	(haddlh),a
	ld	a,CNTL_ADR_HL
	out	(haddhl),a
	ld	a,CNTL_ADR_HH
	out	(haddhh),a
	ld	a,00h
	out	(hdatal),a
	ld	a,8ch
	out	(hdatah),a
  ifndef DOWNLOAD
; Initialize host control register
	ld	a,high (HLT+CF+INCR+INCW+NMIMODE)
	out	(hctrlh),a
	ld	a,low (HLT+CF+INCR+INCW+NMIMODE)
	out	(hctrll),a
; Get start addr of code from interrupt vector table
	ld	a,GDP_RES_VECT_LL
	out	(haddll),a
	ld	a,GDP_RES_VECT_LH
	out	(haddlh),a
	ld	a,GDP_RES_VECT_HL
	out	(haddhl),a
	ld	a,GDP_RES_VECT_HH
	out	(haddhh),a
	in	a,(hdatal)		; Get low start addr
	ld	l,a
	in	a,(hdatah)
	ld	h,a
	push	hl
	in	a,(hdatal)		; Get high start addr
	ld	l,a
	in	a,(hdatah)
	ld	h,a
; Point NMI vector to start of code
	ld	a,NMI_LOW_LL
	out	(haddll),a
	ld	a,NMI_LOW_LH
	out	(haddlh),a
	ld	a,NMI_LOW_HL
	out	(haddhl),a
	ld	a,NMI_LOW_HH
	out	(haddhh),a
	ex	(sp),hl			; Put low start addr
	ld	a,l
	out	(hdatal),a
	ld	a,h
	out	(hdatah),a
	pop	hl			; Put high start addr
	ld	a,l
	out	(hdatal),a
	ld	a,h
	out	(hdatah),a
; Start GDP side of Graphics Manager
	ld	hl,INCR+INCW+NMIMODE+NMI
	ld	a,l
	out	(hctrll),a
	ld	a,h
	out	(hctrlh),a
  endif
; Synchronize host and GDP communication
; Send message 7
	in	a,(hctrll)
	and	~MSGIN
	or	MSGIN7
	out	(hctrll),a
; Wait until GDP send message 7 or timeout
	ld	hl,0
gm_init_wait_msgout7:
	in	a,(hctrll)
	and	MSGOUT
	cp	MSGOUT7
	jr	z,gm_init_1
	dec	hl
	ld	a,l				; Timeout?
	or	h
	ret	z				; return FALSE
	jr	gm_init_wait_msgout7
gm_init_1:
; Send message 0
	in	a,(hctrll)
	and	~MSGIN
	out	(hctrll),a
; Wait until GDP send message 0 or timeout
	ld	hl,0
gm_init_wait_msgout0:
	in	a,(hctrll)
	and	MSGOUT
	cp	MSGOUT0
	jr	z,gm_init_2
	dec	hl
	ld	a,l				; Timeout?
	or	h
	ret	z				; return FALSE
	jr	gm_init_wait_msgout0
gm_init_2:
; Get address of GM communication buffer
	in	a,(haddll)
	ld	l,a
	in	a,(haddlh)
	ld	h,a
	push	hl
	in	a,(haddhl)
	ld	l,a
	in	a,(haddhh)
	ld	h,a
	ld	(_cmdbuf+2),hl
	ex	(sp),hl
	ld	(_cmdbuf),hl
; Argument buffer is command buffer + 16(bit)
	ld	de,16
	add	hl,de
	ld	(_argbuf),hl
	pop	hl
	ld	de,0
	adc	hl,de
	ld	(_argbuf+2),hl
	ld	hl,1			; return TRUE
	ret

	.align	2
; Move <B> words from (HL++) to GDP memory via host port
; B must be in the range of 1 to 128
; Host address register must be set to the first address in GDP memory
; Autoincrement on write must be activated
n2hdata:
	sla	b
	ld	c,hdatah
n2hdata_lp:
	inc	c
	outi
	dec	c
	outi
	Rev_K
	jr	nz,n2hdata_lp
	ret

	.align	2
; Move <DE> words from (HL++) to GDP memory via host port
; DE must be in the range of 1 to 65535
; Host address register must be set to the first address in GDP memory
; Autoincrement on write must be activated
nn2hdata:
	ld	c,hdatah
nn2hdata_lp:
	inc	c
	outi
	dec	c
	outi
	Rev_K
	dec	de
	ld	a,d
	or	e
	jr	nz,nn2hdata_lp
	ret

	.align	2
; Move '0' terminated string from (HL++) to GDP memory via host port
; Host address register must be set to the first address in GDP memory
; Autoincrement on write must be activated
str2hdata:
	ld	a,(hl)
	or	a
	out	(hdatal),a
	Rev_K
	inc	hl
	jr	z,str2hdata_trigger_wr
	ld	a,(hl)
	or	a
str2hdata_trigger_wr:
	out	(hdatah),a
	Rev_K
	inc	hl
	jr	nz,str2hdata
	ret

	.align	2
; Move <B> words from argument buffer via host port to (HL++)
argbuf2n:
	ld	a,(_argbuf)	; Set argument buffer address
	out	(haddll),a
	ld	a,(_argbuf+1)
	out	(haddlh),a
	ld	a,(_argbuf+2)
	out	(haddhl),a
	ld	a,(_argbuf+3)
	out	(haddhh),a
; Rest is done in hdata2n

	.align	2
; Move <B> words from GDP memory via host port to (HL++)
; B must be in the range of 1 to 128
; Host address register must be set to the first address in GDP memory
; Autoincrement on read must be activated
hdata2n:
	sla	b
	ld	c,hdatah
hdata2n_lp:
	inc	c
	ini
	dec	c
	ini
	jr	nz,hdata2n_lp
	ret

	.align	2
; Move <DE> words from argument buffer via host port to (HL++)
argbuf2nn:
	ld	a,(_argbuf)	; Set argument buffer address
	out	(haddll),a
	ld	a,(_argbuf+1)
	out	(haddlh),a
	ld	a,(_argbuf+2)
	out	(haddhl),a
	ld	a,(_argbuf+3)
	out	(haddhh),a
; Rest is done in hdata2nn

	.align	2
; Move <DE> words from GDP memory via host port to (HL++)
; DE must be in the range of 1 to 65535
; Host address register must be set to the first address in GDP memory
; Autoincrement on read must be activated
hdata2nn:
	ld	c,hdatah
hdata2nn_lp:
	inc	c
	ini
	dec	c
	ini
	dec	de
	ld	a,d
	or	e
	jr	nz,hdata2nn_lp
	ret

	.align	2
; Common entry point for void functions with no parameters
common_void_void:
	call	gm_cmd		; Put command in communication buffer
	ld	hl,0		; This function doesn't return a value
	call	gm_sync_cmd	; so call gm_sync_cmd with parameter wait=0
	ret

	.align	2
; Common entry point for void functions with simple parameters
common_void_simple:
	call	gm_cmd		; Put command in communication buffer
				; hadd now points to argument bufferdata
	ParAddr	2		; Get address of parameters
	call	n2hdata		; Move data
	ld	hl,0		; This function doesn't return a value
	call	gm_sync_cmd	; so call gm_sync_cmd with parameter wait=0
	ret

	.align	2
; Common entry point for functions with no parameters and short return value
common_short_void:
	call	gm_cmd		; Put command in communication buffer
	ld	hl,1		; Function return a value so we have to wait
	call	gm_sync_cmd	; for command execution (parameter wait=1)
	in	a,(haddll)	; Get short return value
	ld	l,a
	in	a,(haddlh)
	ld	h,a
	ret

	.align	2
; Common entry point for functions with simple parameters and short return value
common_short_simple:
	call	gm_cmd		; Put command in communication buffer
				; hadd now points to argument bufferdata
	ParAddr	2		; Get address of parameters
	call	n2hdata		; Move data
	ld	hl,1		; Function return a value so we have to wait
	call	gm_sync_cmd	; for command execution (parameter wait=1)
	in	a,(haddll)	; Get short return value
	ld	l,a
	in	a,(haddlh)
	ld	h,a
	ret

	.align	2
; Common entry point for functions with no parameters and long return value
common_long_void:
	call	gm_cmd		; Put command in communication buffer
	ld	hl,1		; Function return a value so we have to wait
	call	gm_sync_cmd	; for command execution (parameter wait=1)
	in	a,(haddll)	; Get long return value
	ld	e,a
	in	a,(haddlh)
	ld	d,a
	in	a,(haddhl)
	ld	c,a
	in	a,(haddhh)
	ld	b,a
	ret

	.align	2
; Common entry point for functions with simple parameters and long return value
common_long_simple:
	call	gm_cmd		; Put command in communication buffer
				; hadd now points to argument bufferdata
	ParAddr	2		; Get address of parameters
	call	n2hdata		; Move data
	ld	hl,1		; Function return a value so we have to wait
	call	gm_sync_cmd	; for command execution (parameter wait=1)
	in	a,(haddll)	; Get long return value
	ld	e,a
	in	a,(haddlh)
	ld	d,a
	in	a,(haddhl)
	ld	c,a
	in	a,(haddhh)
	ld	b,a
	ret

	.align	2
; Common entry point for draw polygon functions
common_poly:
	call	gm_cmd
	Par	2		; Get number of vertices
	ld	a,l		; Store in argument buffer
	out	(hdatal),a
	ld	a,h
	out	(hdatah),a
	Rev_K
	add	hl,hl		; POINT2D = 2 short
	ex	de,hl		; = number of words to move to argument buffer
	Par	4		; Get pointer to vertices
	call	nn2hdata
	ld	hl,0
	call	gm_sync_cmd
	ret

	.align	2
;
; void bitblt(short w, short h, short xs, short ys, short xd, short yd)
;
_bitblt::
	ld	hl,BitBlt	; Function number
	ld	b,6		; Number of words to move into argument buffer
	jp	common_void_simple

	.align	2
;
; void clear_frame_buffer(ulong color)
;
_clear_frame_buffer::
	ld	hl,Clear_Frame_Buffer
	ld	b,2
	jp	common_void_simple
	
	.align	2
;
; void clear_page(ulong color)
;
_clear_page::
	ld	hl,Clear_Page
	ld	b,2
	jp	common_void_simple

	.align	2
;
; void clear_screen(ulong color)
;
_clear_screen::
	ld	hl,Clear_Screen
	ld	b,2
	jp	common_void_simple

	.align	2
;
; short cpw(shortx, short y)
;
_cpw::
	ld	hl,Cpw
	ld	b,2
	jp	common_short_simple

	.align	2
;
; PTR cvxyl(short x, short y)
;
_cvxyl::
	ld	hl,Cvxyl
	ld	b,2
	jp	common_long_simple

	.align	2
;
; short decode_rect(short xleft, short ytop, PTR buf)
;
_decode_rect::
	ld	hl,Decode_Rect
	ld	b,4
	jp	common_short_simple

	.align	2
;
; void delay(short screens)
;
_delay::
	ld	hl,Delay
	ld	b,1
	jp	common_void_simple

	.align	2
;
; short delete_font(short id)
;
_delete_font::
	ld	hl,Delete_Font
	ld	b,1
	jp	common_short_simple

	.align	2
;
; void draw_line(short x1, short y1, short x2, short y2)
;
_draw_line::
	ld	hl,Draw_Line
	ld	b,4
	jp	common_void_simple

	.align	2
;
; void draw_oval(short w, short h, short xleft, short ytop)
;
_draw_oval::
	ld	hl,Draw_Oval
	ld	b,4
	jp	common_void_simple

	.align	2
;
; void draw_ovalarc(short w, short h, short xleft, short ytop,
;			 short theta, short arc)
;
_draw_ovalarc::
	ld	hl,Draw_OvalArc
	ld	b,6
	jp	common_void_simple

	.align	2
;
; void draw_piearc(short w, short h, short xleft, short yleft,
;			short theta, short arc
;
_draw_piearc::
	ld	hl,Draw_PieArc
	ld	b,6
	jp	common_void_simple

	.align	2
;
; void draw_point(short x, short y)
;
_draw_point::
	ld	hl,Draw_Point
	ld	b,2
	jp	common_void_simple

	.align	2
;
; void draw_polyline(short n, POINT2D *vert)
;
_draw_polyline::
	ld	hl,Draw_PolyLine
	jp	common_poly

	.align	2
;
; void draw_rect(short w, short h, short xleft, short ytop)
;
_draw_rect::
	ld	hl,Draw_Rect
	ld	b,4
	jp	common_void_simple

	.align	2
;
; ulong encode_rect(short w, short h, short xleft, short ytop
;			PTR buf, ulong bufsize, ushort scheme)
;
_encode_rect::
	ld	hl,Encode_Rect
	ld	b,9
	jp	common_long_simple

	.align	2
;
; ulong field_extract(PTR gptr, ushort fs)
;
_field_extract::
	ld	hl,Field_Extract
	ld	b,3
	jp	common_long_simple

	.align	2
;
; void field_insert(PTR gptr, short fs, ulong val)
;
_field_insert::
	ld	hl,Field_Insert
	ld	b,5
	jp	common_void_simple

	.align	2
;
; void fill_convex(short n, POINT2D *vert)
;
_fill_convex::
	ld	hl,Fill_Convex
	jp	common_poly

	.align	2
;
; void fill_oval(short w, short h, short xleft, short ytop)
;
_fill_oval::
	ld	hl,Fill_Oval
	ld	b,4
	jp	common_void_simple

	.align	2
;
; void fill_piearc(short w, short h, short xleft, short ytop,
;			short theta, short arc)
;
_fill_piearc::
	ld	hl,Fill_PieArc
	ld	b,6
	jp	common_void_simple

	.align	2
;
; void fill_polygon(short n, POINT2D *vert)
;
_fill_polygon::
	ld	hl,Fill_Polygon
	jp	common_poly

	.align	2
;
; void fill_rect(short w, short h, short xleft, short ytop)
;
_fill_rect::
	ld	hl,Fill_Rect
	ld	b,4
	jp	common_void_simple

	.align	2
;
; void frame_oval(short w, short h, short xleft, short ytop
;			short dx, short dy)
;
_frame_oval::
	ld	hl,Frame_Oval
	ld	b,6
	jp	common_void_simple

	.align	2
;
; void frame_rectl(short w, short h, short xleft, short ytop
;			short dx, short dy)
;
_frame_rect::
	ld	hl,Frame_Rect
	ld	b,6
	jp	common_void_simple

	.align	2
;
; void get_colors(ulong *fcol, ulong *bcol)
;
_get_colors::
	ld	hl,Get_Colors
	call	gm_cmd
	ld	hl,1
	call	gm_sync_cmd
	ld	a,(_argbuf)	; Set argument buffer address
	out	(haddll),a
	ld	a,(_argbuf+1)
	out	(haddlh),a
	ld	a,(_argbuf+2)
	out	(haddhl),a
	ld	a,(_argbuf+3)
	out	(haddhh),a
	Rev_K
	Par	2		; Get address of fcol
	ld	b,2
	call	hdata2n		; Get fcol
	Par	4		; Get address of bcol
	ld	b,2
	call	hdata2n		; Get bcol
	ret

	.align	2
;
; void get_config(CONFIG *config)
;
_get_config::
	ld	hl,Get_Config
	call	gm_cmd
	ld	hl,1
	call	gm_sync_cmd
	Par	2		; Get address of config
	ld	b,45		; Length of struct CONFIG in words
	call	argbuf2n	; Get config structure
	ret

	.align	2
;
; short get_curs_state(void)
;
_get_curs_state::
	ld	hl,Get_Curs_State
	jp	common_short_void
	ret

	.align	2
;
; void get_curs_xy(short *px, short *py)
;
_get_curs_xy::
	ld	hl,Get_Curs_XY
	call	gm_cmd
	ld	hl,1
	call	gm_sync_cmd
	ld	a,(_argbuf)	; Set argument buffer address
	out	(haddll),a
	ld	a,(_argbuf+1)
	out	(haddlh),a
	ld	a,(_argbuf+2)
	out	(haddhl),a
	ld	a,(_argbuf+3)
	out	(haddhh),a
	Rev_K
	Par	2		; Get address of px
	ld	b,1
	call	hdata2n		; Get px
	Par	4		; Get address of py
	ld	b,1
	call	hdata2n		; Get py
	ret

	.align	2
;
; void get_env(ENVIRONMENT *env)
;
_get_env::
	ld	hl,Get_Env
	call	gm_cmd
	ld	hl,1
	call	gm_sync_cmd
	Par	2		; Get address of env
	ld	b,10		; Length of struct ENVIRONMENT in words
	call	argbuf2n	; Get env structure
	ret

	.align	2
;
; short get_fontinfo(short id, FONTINFO *fontinfo)
;
_get_fontinfo::
	ld	hl,Get_FontInfo
	call	gm_cmd
	ParAddr	2
	ld	b,1
	call	n2hdata
	ld	hl,1
	call	gm_sync_cmd
	in	a,(haddll)	; Get short return value
	ld	l,a
	in	a,(haddlh)
	ld	h,a
	push	hl		; and save it
	Par	6		; Get address of fontinfo
	ld	b,29		; Length of struct FONTINFO in words
	call	argbuf2n	; Get fontinfo structure
	pop	hl		; Restore return parameter
	ret

	.align	2
;
; short get_modeinfo(short index, MODEINFO *modeinfo)
;
_get_modeinfo::
	ld	hl,Get_ModeInfo
	call	gm_cmd
	ParAddr	2
	ld	b,1
	call	n2hdata
	ld	hl,1
	call	gm_sync_cmd
	in	a,(haddll)	; Get short return value
	ld	l,a
	in	a,(haddlh)
	ld	h,a
	push	hl		; and save it
	Par	6		; Get address of modeinfo
	ld	b,20		; Length of struct MODEINFO in words
	call	argbuf2n	; Get modeinfo structure
	pop	hl		; Resore return parameter
	ret

	.align	2
;
; ulong get_nearest_color(uchar r, uchar g, uchar b, uchar i)
;
_get_nearest_color::
	ld	hl,Get_Nearest_Color
	ld	b,4
	jp	common_long_simple

	.align	2
;
; void get_offscreen_memory(short num_blocks, OFFSCREEN_AREA *offscreen)
;
_get_offscreen_memory::
	ld	hl,Get_Offscreen_Memory
	call	gm_cmd
	ParAddr	2
	ld	b,1
	call	n2hdata
	ld	hl,1
	call	gm_sync_cmd
	Par	2		; Get num_blocks
	ld	a,l
	add	a,a		; * Length of struct OFFSCREEN_AREA in words
	add	a,a
	ld	b,a
	Par	4		; Get address of offscreen structure array
	call	argbuf2n	; Get offscreen structure array
	ret

	.align	2
;
; void get_palet(short palet_size, PALET *palet)
;
_get_palet::
	ld	hl,Get_Palet
	call	gm_cmd
	ParAddr	2
	ld	b,1
	call	n2hdata
	ld	hl,1
	call	gm_sync_cmd
	Par	2		; Get palet_size
	add	hl,hl		; * 2 (Length of struct PALET in words)
	ex	de,hl
	Par	4		; Get address of palet
	call	argbuf2nn	; Get palet structure array
	ret

	.align	2
;
; short get_palet_entry(long index, uchar *r, uchar *g, uchar *b, uchar *i)
;
_get_palet_entry::
	ld	hl,Get_Palet_Entry
	call	gm_cmd
	ParAddr	2
	ld	b,2
	call	n2hdata
	ld	hl,1
	call	gm_sync_cmd
	in	a,(haddll)	; Get short return value
	ld	l,a
	in	a,(haddlh)
	ld	h,a
	push	hl
	ld	a,(_argbuf)	; Set argument buffer address
	out	(haddll),a
	ld	a,(_argbuf+1)
	out	(haddlh),a
	ld	a,(_argbuf+2)
	out	(haddhl),a
	ld	a,(_argbuf+3)
	out	(haddhh),a
	Par	8		; Get address of r
	in	a,(hdatal)	; Get red value
	ld	(hl),a		; Save red value
	Par	10		; Get address of g
	in	a,(hdatah)	; Get green value
	ld	(hl),a		; Save green value
	Par	12		; Get address of b
	in	a,(hdatal)	; Get blue value
	ld	(hl),a		; Save blue value
	Par	14		; Get address of i
	in	a,(hdatah)	; Get intensity value
	ld	(hl),a		; Save intensity value
	pop	hl		; Restore return parameter
	ret

	.align	2
;
; ulong get_pixel(short x, short y)
;
_get_pixel::
	ld	hl,Get_Pixel
	ld	b,2
	jp	common_long_simple
	ret

	.align	2
;
; ulong get_pmask(void)
;
_get_pmask::
	ld	hl,Get_PMask
	jp	common_long_void

	.align	2
;
; ushort get_ppop(void)
;
_get_ppop::
	ld	hl,Get_PPop
	jp	common_short_void
	ret

	.align	2
;
; short get_textattr(char *pcontrol, short count, short *val)
;
_get_textattr::
	ld	hl,Get_TextAttr
	call	gm_cmd
	ld	de,(_argbuf)		; Calc pcontrol and val
	ld	bc,(_argbuf+2)		; in GDP memory
	ld	hl,80			; val = _argbuf + PTR + short + PTR
	add	hl,de
	ex	de,hl
	ld	hl,0
	adc	hl,bc
	ld	b,h
	ld	c,l
	push	bc			; Save val
	push	de
	Par	8			; pcontrol = val + count*short
	add	hl,hl			; count * 16
	add	hl,hl
	add	hl,hl
	add	hl,hl
	add	hl,de			; + val
	ex	de,hl
	ld	hl,0
	adc	hl,bc
	push	hl			; pcontrol is now in (sp):de
	ex	de,hl			; Store pcontrol in argument buffer
	ld	a,l
	out	(hdatal),a
	ld	a,h
	out	(hdatah),a
	pop	hl
	ld	a,l
	out	(hdatal),a
	ld	a,h
	out	(hdatah),a
	Par	8			; Store count in argument buffer
	ld	a,l
	out	(hdatal),a
	ld	a,h
	out	(hdatah),a
	Par	0			; Store val in argument buffer
	ld	a,l
	out	(hdatal),a
	ld	a,h
	out	(hdatah),a
	Par	2
	ld	a,l
	out	(hdatal),a
	ld	a,h
	out	(hdatah),a
	Rev_K
	Par	8			; move host address value to *pcontrol
get_textattr_lp1:
	ld	a,l
	or	h
	jr	z,get_textattr_1
	out	(hdatah),a		; using dummy data write
	Rev_K
	dec	hl
	jr	get_textattr_lp1
get_textattr_1:
	Par	6			; Store control string in argument buffer
	call	str2hdata
	ld	hl,1			; Trigger command execution and
	call	gm_sync_cmd		; wait for return value
	in	a,(haddll)		; Get short return value
	ld	l,a
	in	a,(haddlh)
	ld	h,a
	pop	de			; Set host address to val
	ld	a,e
	out	(haddll),a
	ld	a,d
	out	(haddlh),a
	pop	de
	ld	a,e
	out	(haddhl),a
	ld	a,d
	out	(haddhh),a
	Rev_K
	push	hl			; Save return value
	Par	6			; Get count text attributs
	ld	a,l
	or	h
	jr	get_textattr_e		; nothing to move
	ex	de,hl
	Par	8
	call	hdata2nn
get_textattr_e:
	pop	hl			; Restore return parameter
	ret

	.align	2
;
; void get_text_xy(short *x, short *y)
;
_get_text_xy::
	ld	hl,Get_Text_XY
	call	gm_cmd
	ld	hl,1
	call	gm_sync_cmd
	ld	a,(_argbuf)	; Set argument buffer address
	out	(haddll),a
	ld	a,(_argbuf+1)
	out	(haddlh),a
	ld	a,(_argbuf+2)
	out	(haddhl),a
	ld	a,(_argbuf+3)
	out	(haddhh),a
	Rev_K
	Par	2		; Get address of x
	ld	b,1
	call	hdata2n		; Get x
	Par	4		; Get address of y
	ld	b,1
	call	hdata2n		; Get y
	ret

	.align	2
;
; short get_transp(void)
;
_get_transp::
	ld	hl,Get_Transp
	jp	common_short_void

	.align	2
;
; PTR get_vector(short trapnum)
;
_get_vector::
	ld	hl,Get_Vector
	ld	b,1
	jp	common_long_simple

	.align	2
;
; short get_windowing(void)
;
_get_windowing::
	ld	hl,Get_Windowing
	jp	common_short_void

	.align	2
;
; short get_wksp(PTR *addr, ulong *pitch)
;
_get_wksp::
	ld	hl,Get_Wksp
	call	gm_cmd
	ld	hl,1
	call	gm_sync_cmd
	in	a,(haddll)	; Get short return value
	ld	l,a
	in	a,(haddlh)
	ld	h,a
	push	hl
	ld	a,(_argbuf)	; Set argument buffer address
	out	(haddll),a
	ld	a,(_argbuf+1)
	out	(haddlh),a
	ld	a,(_argbuf+2)
	out	(haddhl),a
	ld	a,(_argbuf+3)
	out	(haddhh),a
	Rev_K
	Par	4		; Get address of addr
	ld	b,2
	call	hdata2n		; Get addr
	Par	6		; Get address of pitch
	ld	b,2
	call	hdata2n		; Get pitch
	pop	hl		; Restore return value
	ret

	.align	2
;
; PTR gsp_calloc(ulong num, ulong size)
;
_gsp_calloc::
	ld	hl,Gsp_CAlloc
	ld	b,4
	jp	common_long_simple

	.align	2
;
; gsp_free(PTR ptr)
;
_gsp_free::
	ld	hl,Gsp_Free
	ld	b,2
	jp	common_void_simple

	.align	2
;
; PTR gsp_malloc(ulong size)
;
_gsp_malloc::
	ld	hl,Gsp_MAlloc
	ld	b,2
	jp	common_long_simple

	.align	2
;
; void gsp_minit(void)
;
_gsp_minit::
	ld	hl,Gsp_MInit
	jp	common_void_void

	.align	2
;
; PTR gsp_realloc(PTR ptr, ulong size)
;
_gsp_realloc::
	ld	hl,Gsp_ReAlloc
	ld	b,4
	jp	common_long_simple

	.align	2
;
; void gsp2gsp(PTR src, PTR dst, ulong length)
;
_gsp2gsp::
	ld	hl,Gsp2Gsp
	ld	b,6
	jp	common_void_simple

	.align	2
;
; short in_font(short start_code, short end_code)
;
_in_font::
	ld	hl,In_Font
	ld	b,2
	jp	common_short_simple

	.align	2
;
; short install_font(PTR pfont)
;
_install_font::
	ld	hl,Install_Font
	ld	b,2
	jp	common_short_simple

	.align	2
;
; short install_hard_font(short n)
;
_install_hard_font::
	ld	hl,Install_Hard_Font
	ld	b,1
	jp	common_short_simple

	.align	2
;
; void init_palet(void)
;
_init_palet::
	ld	hl,Init_Palet
	jp	common_void_void

	.align	2
;
; void init_text(void)
;
_init_text::
	ld	hl,Init_Text
	jp	common_void_void

	.align	2
;
; short lmo(long n)
;
_lmo::
	ld	hl,Lmo
	ld	b,2
	jp	common_short_simple

	.align	2
;
; void move_pixel(short xs, short ys, short xd, short yd)
;
_move_pixel::
	ld	hl,Move_Pixel
	ld	b,4
	jp	common_void_simple

	.align	2
;
; short page_busy(void)
;
_page_busy::
	ld	hl,Page_Busy
	jp	common_short_void

	.align	2
;
; void page_flip(short disp, short draw)
;
_page_flip::
	ld	hl,Page_Flip
	ld	b,2
	jp	common_void_simple

	.align	2
;
; void patnfill_convex(short n, POINT2D *vert)
;
_patnfill_convex::
	ld	hl,PatnFill_Convex
	jp	common_poly

	.align	2
;
; void patnfill_oval(short w, short h, short xleft, short ytop)
;
_patnfill_oval::
	ld	hl,PatnFill_Oval
	ld	b,4
	jp	common_void_simple

	.align	2
;
; void patnfill_piearc(short w, short h, short xleft, short ytop
;			short theta, short arc)
;
_patnfill_piearc::
	ld	hl,PatnFill_PieArc
	ld	b,6
	jp	common_void_simple

	.align	2
;
; void patnfill_polygon(short n, POINT2D *vert)
;
_patnfill_polygon::
	ld	hl,PatnFill_Polygon
	jp	common_poly

	.align	2
;
; void patnfill_rect(short w, short h, short xleft, short ytop)
;
_patnfill_rect::
	ld	hl,PatnFill_Rect
	ld	b,4
	jp	common_void_simple

	.align	2
;
; void patnframe_oval(short w, short h, short xleft, short ytop
;			short dx, short dy)
;
_patnframe_oval::
	ld	hl,PatnFrame_Oval
	ld	b,6
	jp	common_void_simple

	.align	2
;
; void patnframe_rect(short w, short h, short xleft, short ytop
;			short dx, short dy)
;
_patnframe_rect::
	ld	hl,PatnFrame_Rect
	ld	b,6
	jp	common_void_simple

	.align	2
;
; void patnpen_line(short x1, short y1, short x2, short y2)
;
_patnpen_line::
	ld	hl,PatnPen_Line
	ld	b,4
	jp	common_void_simple

	.align	2
;
; void patnpen_ovalarc(short w, short h, short xleft, short ytop,
;			short theta, short arc)
;
_patnpen_ovalarc::
	ld	hl,PatnPen_OvalArc
	ld	b,6
	jp	common_void_simple

	.align	2
;
; void patnpen_piearc(short w, short h, short xleft, short ytop,
;			short theta, short arc)
;
_patnpen_piearc::
	ld	hl,PatnPen_PieArc
	ld	b,6
	jp	common_void_simple

	.align	2
;
; void patnpen_point(short x, short y)
;
_patnpen_point::
	ld	hl,PatnPen_Point
	ld	b,2
	jp	common_void_simple

	.align	2
;
; void patnpen_polyline(short n, POINT2D *vert)
;
_patnpen_polyline::
	ld	hl,PatnPen_PolyLine
	jp	common_poly

	.align	2
;
; ulong peek_breg(short breg)
;
_peek_breg::
	ld	hl,Peek_BReg
	ld	b,1
	jp	common_long_simple

	.align	2
;
; void pen_line(short x1, short y1, short x2, short y2)
;
_pen_line::
	ld	hl,Pen_Line
	ld	b,4
	jp	common_void_simple

	.align	2
;
; void pen_ovalarc(short w, short h, short xleft, short ytop,
;			short theta, short arc)
;
_pen_ovalarc::
	ld	hl,Pen_OvalArc
	ld	b,6
	jp	common_void_simple

	.align	2
;
; void pen_piearc(short w, short h, short xleft, short ytop,
;			short theta, short arc)
;
_pen_piearc::
	ld	hl,Pen_PieArc
	ld	b,6
	jp	common_void_simple

	.align	2
;
; void pen_point(short x, short y)
;
_pen_point::
	ld	hl,Pen_Point
	ld	b,2
	jp	common_void_simple

	.align	2
;
; void pen_polyline(short n, POINT2D *vert)
;
_pen_polyline::
	ld	hl,Pen_PolyLine
	jp	common_poly

	.align	2
;
; void poke_breg(short breg, ulong val)
;
_poke_breg::
	ld	hl,Poke_BReg
	ld	b,3
	jp	common_void_simple

	.align	2
;
; void put_pixel(ulong val, short x, short y)
;
_put_pixel::
	ld	hl,Put_Pixel
	ld	b,4
	jp	common_void_simple

	.align	2
;
; short rmo(ulong n)
;
_rmo::
	ld	hl,Rmo
	ld	b,1
	jp	common_short_simple

	.align	2
;
; void seed_fill(short x, short y)
;
_seed_fill::
	ld	hl,Seed_Fill
	ld	b,2
	jp	common_void_simple

	.align	2
;
; void seed_patnfill(short x, short y)
;
_seed_patnfill::
	ld	hl,Seed_PatnFill
	ld	b,2
	jp	common_void_simple

	.align	2
;
; short select_font(short id)
;
_select_font::
	ld	hl,Select_Font
	ld	b,1
	jp	common_short_simple

	.align	2
;
; void set_bcolor(ulong color)
;
_set_bcolor::
	ld	hl,Set_BColor
	ld	b,2
	jp	common_void_simple

	.align	2
;
; void set_clip_rect(ushort w, ushort h, short xleft, short ytop)
;
_set_clip_rect::
	ld	hl,Set_Clip_Rect
	ld	b,4
	jp	common_void_simple

	.align	2
;
; void set_colors(ulong fcolor, ulong bcolor)
;
_set_colors::
	ld	hl,Set_Colors
	ld	b,4
	jp	common_void_simple

	.align	2
;
; short set_config(short graphics_mode, short init_draw)
;
_set_config::
	ld	hl,Set_Config
	ld	b,2
	jp	common_short_simple

	.align	2
;
; void set_curs_shape(PTR shape)
;
_set_curs_shape::
	ld	hl,Set_Curs_Shape
	ld	b,2
	jp	common_void_simple

	.align	2
;
; void set_curs_state(short enab)
;
_set_curs_state::
	ld	hl,Set_Curs_State
	ld	b,1
	jp	common_void_simple

	.align	2
;
; void set_curs_xy(short x, short y)
;
_set_curs_xy::
	ld	hl,Set_Curs_XY
	ld	b,2
	jp	common_void_simple

	.align	2
;
; void set_cursattr(ulong shape_c, ulong mask_c, ushort shape_a, ushort mask_a)
;
_set_cursattr::
	ld	hl,Set_CursAttr
	ld	b,6
	jp	common_void_simple

	.align	2
;
; void set_draw_origin(short x, short y)
;
_set_draw_origin::
	ld	hl,Set_Draw_Origin
	ld	b,2
	jp	common_void_simple

	.align	2
;
; void set_dstbm(PTR baseaddr, short pitch, short xext, short yext, short psize)
;
_set_dstbm::
	ld	hl,Set_DstBm
	ld	b,6
	jp	common_void_simple

	.align	2
;
; set_fcolor(ulong color)
;
_set_fcolor::
	ld	hl,Set_FColor
	ld	b,2
	jp	common_void_simple

	.align	2
;
; set_hard_patn(short patn_nr)
;
_set_hard_patn::
	ld	hl,Set_Hard_Patn
	ld	b,1
	jp	common_void_simple

	.align	2
;
; void set_hard_curs_shape(short id)
;
_set_hard_curs_shape::
	ld	hl,Set_Hard_Curs_Shape
	ld	b,1
	jp	common_void_simple

	.align	2
;
; void set_palet(long count, long index, PALET *palet)
;
_set_palet::
	ld	hl,Set_Palet
	call	gm_cmd
	ParAddr	2
	ld	b,4
	call	n2hdata
; Move count PALET entries from palet[index++] to argument buffer
	Par	6		; Calc &palet[index]
	add	hl,hl		; sizeof(PALET) is 4
	add	hl,hl
	ex	de,hl
	Par	10		; Get &palet[0]
	add	hl,de		; Now we have &palet[index] in hl
	ex	de,hl
	Par	2		; Get count
	add	hl,hl		; * 2 = number of words to move
	ex	de,hl
	call	nn2hdata	; Do move
	ld	hl,0
	call	gm_sync_cmd	; and start command execution
	ret

	.align	2
;
; short set_palet_entry(long count, uchar r, uchar g, uchar b, uchar i)
;
_set_palet_entry::
	ld	hl,Set_Palet_Entry
	ld	b,6
	jp	common_short_simple

	.align	2
;
; void set_patn(PATTERN *ppatn)
;
_set_patn::
	ld	hl,Set_Patn
	call	gm_cmd
	Par	2
	ld	b,5		; PATTERN structure length is 5 words
	call	n2hdata
	ld	hl,0
	call	gm_sync_cmd
	ret

	.align	2
;
; void set_pensize(short x, short y)
;
_set_pensize::
	ld	hl,Set_PenSize
	ld	b,2
	jp	common_void_simple

	.align	2
;
; set_pmask(ulong pmask)
;
_set_pmask::
	ld	hl,Set_PMask
	ld	b,2
	jp	common_void_simple

	.align	2
;
; set_ppop(short ppop)
;
_set_ppop::
	ld	hl,Set_PPop
	ld	b,1
	jp	common_void_simple

	.align	2
;
; void set_srcbm(PTR baseaddr, short pitch, short xext, short yext, short psize)
;
_set_srcbm::
	ld	hl,Set_SrcBm
	ld	b,6
	jp	common_void_simple

	.align	2
;
; void set_text_xy(short x, short y)
;
_set_text_xy::
	ld	hl,Set_Text_XY
	ld	b,2
	jp	common_void_simple

	.align	2
;
; short set_textattr(char *pcontrol, short count, short *val)
;
_set_textattr::
	ld	hl,Set_TextAttr
	call	gm_cmd
	ld	de,(_argbuf)		; Calc pcontrol and val
	ld	bc,(_argbuf+2)		; in GDP memory
	ld	hl,80			; val = _argbuf + PTR + short + PTR
	add	hl,de
	ex	de,hl
	ld	hl,0
	adc	hl,bc
	ld	b,h
	ld	c,l
	push	bc			; Save val
	push	de
	Par	8			; pcontrol = val + count*short
	add	hl,hl			; count * 16
	add	hl,hl
	add	hl,hl
	add	hl,hl
	add	hl,de			; + val
	ex	de,hl
	ld	hl,0
	adc	hl,bc
	push	hl			; pcontrol is now in (sp):de
	ex	de,hl			; Store pcontrol in argument buffer
	ld	a,l
	out	(hdatal),a
	ld	a,h
	out	(hdatah),a
	pop	hl
	ld	a,l
	out	(hdatal),a
	ld	a,h
	out	(hdatah),a
	Par	8			; Store count in argument buffer
	ld	a,l
	out	(hdatal),a
	ld	a,h
	out	(hdatah),a
	pop	hl			; Store val in argument buffer
	ld	a,l
	out	(hdatal),a
	ld	a,h
	out	(hdatah),a
	pop	hl
	ld	a,l
	out	(hdatal),a
	ld	a,h
	out	(hdatah),a
	Rev_K
	Par	4			; Store count val[] data
	ld	a,l
	or	h
	jr	z,set_textattr_0	; nothing to do
	ex	de,hl
	Par	6
	call	nn2hdata
set_textattr_0:
	Par	2			; Store control string in argument buffer
	call	str2hdata
	ld	hl,1			; Trigger command execution and
	call	gm_sync_cmd		; wait for return value
	in	a,(haddll)		; Get short return value
	ld	l,a
	in	a,(haddlh)
	ld	h,a
	ret

	.align	2
;
; PTR set_vector(short trapnum, PTR gptr)
;
_set_vector::
	ld	hl,Set_Vector
	ld	b,3
	jp	common_long_simple

	.align	2
;
; set_windowing(short mode)
;
_set_windowing::
	ld	hl,Set_Windowing
	ld	b,1
	jp	common_void_simple

	.align	2
;
; void set_wksp(PTR addr, ulong pitch)
;
_set_wksp::
	ld	hl,Set_Wksp
	ld	b,4
	jp	common_void_simple

	.align	2
;
; void styled_line(short x1, short y1, short x2, short y2,
;			long style, short mode)
;
_styled_line::
	ld	hl,Styled_Line
	ld	b,7
	jp	common_void_simple

	.align	2
;
; void styled_oval(short w, short h, short xleft, short ytop,
;			long style, short mode)
;
_styled_oval::
	ld	hl,Styled_Oval
	ld	b,7
	jp	common_void_simple

	.align	2
;
; void styled_ovalarc(short w, short h, short xleft, short ytop,
;			short theta, short arc, long style, short mode)
;
_styled_ovalarc::
	ld	hl,Styled_OvalArc
	ld	b,9
	jp	common_void_simple

	.align	2
;
; void styled_piearc(short w, short h, short xleft, short ytop,
;			short theta, short arc, long style, short mode)
;
_styled_piearc::
	ld	hl,Styled_PieArc
	ld	b,9
	jp	common_void_simple

	.align	2
;
; void swap_bm(void)
;
_swap_bm::
	ld	hl,Swap_Bm
	jp	common_void_void

	.align	2
;
; short text_out(shortx, short y, char *s)
;
_text_out::
	ld	hl,Text_Out
	call	gm_cmd
	ParAddr	2
	ld	b,2
	call	n2hdata
	Par	6		; Put string in argument buffer
text_out_1:
	call	str2hdata
	ld	hl,1		; Trigger command execution and
	call	gm_sync_cmd	; wait for return value
	in	a,(haddll)	; Get short return value
	ld	l,a
	in	a,(haddlh)
	ld	h,a
	ret

	.align	2
;
; short text_outp(char *s)
;
_text_outp::
	ld	hl,Text_OutP
	call	gm_cmd
	Par	2
	jr	text_out_1

	.align	2
;
; short text_width(char *s)
;
_text_width::
	ld	hl,Text_Width
	call	gm_cmd
	Par	2
	jr	text_out_1

	.align	2
;
; void transp_off(void)
;
_transp_off::
	ld	hl,Transp_Off
	jp	common_void_void

	.align	2
;
; void transp_on(void)
;
_transp_on::
	ld	hl,Transp_On
	jp	common_void_void

	.align	2
;
; void wait_scan(short line)
;
_wait_scan::
	ld	hl,Wait_Scan
	ld	b,1
	jp	common_void_simple

	.align	2
;
; void zoom_rect(short ws, short hs, short xs, short ys,
;		 short wd, short hd, short xd, short yd, PTR rowbuf)
;
_zoom_rect::
	ld	hl,Zoom_Rect
	ld	b,10
	jp	common_void_simple

	.align	2
;
; void host2gsp(HPTR hptr, PTR gptr, ushort len, short swizzle)
;
_host2gsp::
	call	wait_msgout0	; wait if command is still in progress
	Par	4		; Set host address register to gptr
	ld	a,l
	out	(haddll),a
	ld	a,h
	out	(haddlh),a
	Par	6
	ld	a,l
	out	(haddhl),a
	ld	a,h
	out	(haddhh),a
	Rev_K
	Par	8		; Len is in byte
	srl	h		; We need in word
	rr	l
	jr	nc,host2gsp_even
	inc	hl		; it's 1 byte to much.... oh,oh
host2gsp_even:
	ex	de,hl
	Par	2
	call	nn2hdata
	ret

	.align	2
;
; void gsp2host(PTR gptr, HPTR hptr, ushort len, short swizzle)
;
_gsp2host::
	call	wait_msgout0	; wait if command is still in progress
	Par	2		; Set host address register to gptr
	ld	a,l
	out	(haddll),a
	ld	a,h
	out	(haddlh),a
	Par	4
	ld	a,l
	out	(haddhl),a
	ld	a,h
	out	(haddhh),a
	Rev_K
	Par	8		; Len is in byte
	srl	h		; We need in word
	rr	l
	jr	nc,gsp2host_even
	inc	hl		; it's 1 byte to much.... oh,oh
gsp2host_even:
	ex	de,hl
	Par	6
	call	hdata2nn
	ret

	.bseg
_argbuf:	.ds	4
_cmdbuf:	.ds	4

	.end
