Program Grafik;
{ unterstuetzt die GDP9366 Emulation der GDP340-Karte }

Type  Str10   = String[10];
      Str255  = String[255];
      Feld1   = Array [1..10] of Real;
      Feld2   = Array [1..10] of Integer;

Const
      STATUS = $00;         {Register der GDP9366 Emulation}
      CMD    = $00;
      CTRL1  = $10;
      CTRL2  = $20;
      CSIZE  = $30;
      DELTAX = $40;
      DELTAY = $50;
      XLSB   = $60;
      YLSB   = $70;
      COLOR  = $80;
      SCROLL = $90;
      RAM    = $a0;
      PAGESEL= $b0;

      HSTADRLl = $D7;
      HSTADRLh = $D6;
      HSTADRHl = $D5;
      HSTADRHh = $D4;
      HSTCTRLl = $D3;
      HSTCTRLh = $D2;
      HSTDATAl = $D1;
      HSTDATAh = $D0;

{$I SINUS.DAT}
{Sinustabelle}

Var
      Meldung    : Str255;
      Statfeld   : Feld1;

      Datei      : File;

      I,J,K,           {Zaehler}
      XPos,YPos,       {Irgendwelche Positionen}
      KX1,KY1,         {Koordinatenursprung Nr. 1}
      VX1,VY1,         {X- und Y-Achse}
      SX1,SY1,         {Scalingfaktoren X,Y}
      KX2,KY2,         {Koordinatenursprung Nr. 2}
      VX2,VY2,         {X- und Y-Achse}
      SX2,SY2          {Scalingfaktoren X,Y}
               : Integer;

      MX1,MY1,        {X und Y Maximalwert}
      MX2,MY2,        {X und Y Maximalwert}
      a,s,t,Deltat,v
               : Real;

{GDP Proceduren:
     Balken         zeichnet eine Balkengrafik
     ClrGDP         loescht die gewaehle Seite des GDP9366
     ClrGDPXY       ClrGDP und ClrXY zusammen
     ClrXY          setzt Register X und Y des EF9366 auf 0
     GDPready       ueberprueft, ob der EF9366 fuer einen neuen Befehl bereit ist
     GetXY          holt die aktuelle X,Y-Position vom EF9366
     GoXY           setzt im EF9366 die X,Y-Register
     InitGDP        initialisiert den EF9366
     Koordi         zeichnet ein Koordinatenkreuz mit Bezeichnung
     KoorPunkt      zeichnet einen Punkt im Koordinatensystem
     Kreis          zeichnet einen Kreis mit bestimmten Radius
     LVektor        zeichnet einen Vektor zur bestimmten X,Y-Position
     NegYAchse      zeichnet negative Y-Achse eines Koordinatensystems
     Punkt          zeichnet einen Punkt
     RMWon          schaltet read-modify-write-Modus an
     RMWoff         schaltet read-modify-write-Modus aus
     Scrollen       schreibt Werte ins SCROLL-Register
     SelChar        waehlt eine Schriftart
     SelDown        Schreibstift wird abgesetzt
     SelEraser      selektiert als Schreibstift Eraser
     SelPage        schaltet zwischen den Seiten der GDP9366 um
     SelPen         selektiert als Schreibstift Pen
     SelType        selektiert die Strichart eines Vektors
     SelSize        waehlt die Schriftgroesse aus
     SelUp          Schreibstift wird hochgesetzt
     SVektor        zeichnet einen Kurzvektor
     Torte          zeichnet eine Tortengrafik
     VBready        testet ob der EF9366 vertical blanking ausfuehrt
     WriteGDP       schreibt ASCII-Zeichen zur GDP9366
     WriteUpcaseGDP schreibt ASCII-Zeichen in Grossbuchstaben zur GDP9366
     WritelnGDP     schreibt ASCII-Zeichen zur GDP9366 mit "CR"
     WVektor        zeichnet einen Vektor mit Winkel und X,Y-Auslenkung
     XVektor        zeichnet einen Vektor parallel zur X-Achse
     YVektor        zeichnet einen Vektor parallel zur Y-Achse

GDP  Function:
     Up             Schreibstift oben?
     Down           Schreibstift unten?
     Eraser         ist Eraser gewaehlt?
     Pen            ist Pen gewaehlt?
     Msin           schnelle Ermittlung von sin(X);
     Mcos           schnelle Ermittlung von Cos(X);
     OutRange       ausserhalb des Bildschirms?
     VideoRam       liefert den Inhalt des adressierten Videorams
}

{$I GDPEMU.PAS enthaelt die Grundprozeduren}

Procedure Kreis(Radius : Byte);
{Zeichnet einen Kreis mit def. Radius}
Var Winkel,
    AltX,AltY,
    NeuX,NeuY  : Integer;
    Faktor     : Byte;
Begin
  Case Radius of
     0..15  : Faktor:= 4;
    16..25  : Faktor:= 3;
    26..255 : Faktor:= 1;
  End; {Case}
  Winkel:= 0;
  GetXY(AltX,AltY);
  Repeat
    NeuX:= Round(Mcos(Winkel) * Radius * 1.6) + AltX;
    NeuY:= Round(Msin(Winkel) * Radius) + AltY;
    GoXY(NeuX,NeuY);
    Punkt;
    Winkel:= Winkel + Faktor;
  until Winkel=360;
  GoXY(AltX,AltY);
End; {Kreis}

Procedure Torte(Radius,Anzahl : Byte; Var Statfeld : Feld1);
{Zeichnet einen Kreis mit def. Radius und traegt stat. Verteilungen ein}
{Die Werte muessen vorher im Statfeld eingetragen werden (Type Real)}
{Die Anzahl der Werte wird mit Anzahl uebergeben}
Var Winkel,
    AltX,AltY,
    NeuX,NeuY  : Integer;
    Summe      : Real;
    StrLaenge,
    K          : Byte;
    Winkelfeld : Feld2;
    Meldung    : Str255;
    Nr         : Str10;
Begin
  Summe:= 0;
  For K:=1 to Anzahl do Summe:= Summe + Statfeld[K];
  Winkelfeld[1]:= Round(360 * Statfeld[1] / Summe);
  For K:=2 to Anzahl do Winkelfeld[K]:= Round(360 * Statfeld[K] / Summe + Winkelfeld[K-1]);
  GetXY(AltX,AltY);
  Kreis(Radius);
  NeuX:= Round(Radius * 1.6) + AltX;
  LVektor(NeuX,AltY);
  GoXY(AltX,AltY);
  For K:=1 to Anzahl-1 do {Verteilungen einzeichnen}
    Begin
      NeuX:= Round(Mcos(Winkelfeld[K]) * Radius * 1.6) + AltX;
      NeuY:= Round(Msin(Winkelfeld[K]) * Radius) + AltY;
      LVektor(NeuX,NeuY);
      GoXY(AltX,AltY);
    End; {For}
  For K:=1 to Anzahl do {Torte bezeichnen}
    Begin
      If K>1 then
        Winkel:= Winkelfeld[K-1] + ((Winkelfeld[K] - Winkelfeld[K-1]) div 2)
      Else
        Winkel:= Winkelfeld[K] div 2;
      NeuX:= Round(Mcos(Winkel) * Radius * 1.6) + AltX;
      NeuY:= Round(Msin(Winkel) * Radius) + AltY;
      Str(Statfeld[K]:13:2,Meldung);
      While (Meldung[1]=' ') do delete(Meldung,1,1);
      Str(K:1,Nr);
      Meldung:= '(' + Nr + ') ' + Meldung;
      StrLaenge:= Length(Meldung);
      Case Winkel of
         0.. 45 : GoXY(NeuX+2,NeuY+2);
        45.. 89 : GoXY(NeuX,NeuY+2);
        90..180 : GoXY(NeuX - 8 * StrLaenge,NeuY);
       181..270 : GoXY(NeuX - 8 * StrLaenge,NeuY-8);
       271..315 : GoXY(NeuX,NeuY-10);
       316..359 : GoXY(NeuX+2,NeuY-8);
      End; {Case}
      WriteGDP(Meldung);
    End; {For}
  GoXY(AltX,AltY);
End; {Torte}

Procedure Balken(Anzahl : Byte; Var Statfeld : Feld1);
{Zeichnet ein Balkendiagramm mit Werten in Statfeld und def. Anzahl}
Var Maxwert : Real;
    YScale  : Byte;
    StrLaenge,
    K       : Byte;
    YFeld   : Feld2;
    XPos,
    YPos    : Integer;
    Meldung : Str255;
    Nr      : Str10;
Begin
  Maxwert:= Statfeld[1];
  For K:=2 to Anzahl do if Maxwert<Statfeld[K] then Maxwert:=Statfeld[K];
  For K:=1 to Anzahl do YFeld[K]:= Round(200 * Statfeld[K] / Maxwert);
  YScale:= 500 div (Anzahl);
  GoXY(500,0);
  LVektor(0,0);
  For K:=1 to Anzahl do
    Begin
      YVektor(YFeld[K]);
      Str(Statfeld[K]:13:2,Meldung);
      While (Meldung[1]=' ') do delete(Meldung,1,1);
      Str(K:1,Nr);
      Meldung:= '(' + Nr + ') ' + Meldung;
      StrLaenge:= Length(Meldung);
      GetXY(XPos,YPos);
      GoXY(XPos + ((YScale - (StrLaenge*6)) div 2),Ypos + 2);
      WriteGDP(Meldung);
      GoXY(XPos,YPos);
      XVektor(YScale);
      GetXY(XPos,YPos);
      LVektor(XPos,0);
    End; {For}
End; {Balken}

Begin
  SelPage(0,0);
  InitGDP;
  RMWoff;
  SelPage(1,1);
  ClrGDP;
  SelPage(2,2);
  ClrGDP;
  SelPage(3,3);
  ClrGDP;
  SelPage(0,0);
  Statfeld[1]:= 165; {Bus 8}
  Statfeld[2]:= 328; {Netzteil}
  Statfeld[3]:= 549; {3 Zoll-Floppy}
  Statfeld[4]:= 798; {CPU 85-SC}
  Statfeld[5]:= 698; {FDC4}
  Statfeld[6]:= 899; {1M 256 K}
  Statfeld[7]:= 649; {Video 80}
  Statfeld[8]:= 498; {DIN-Tast}
  SelSize(2,2);
  GoXY(20,220);
  WriteGDP('Preise fuer einen ELZET 80 - Computer');
  SelSize(1,1);
  GoXY(0,205);
  WritelnGDP('1 = Busplatine BUS 8');
  WritelnGDP('2 = Netzteil');
  WritelnGDP('3 = 3 Zoll Floppy');
  WritelnGDP('4 = CPU - Platine CPU 85 SC');
  WritelnGDP('5 = Floppy-Controller FDC4');
  WritelnGDP('6 = Speicherkarte 1M 256K');
  WritelnGDP('7 = Terminalkarte Video 80F');
  WritelnGDP('8 = Tastatur DIN-Tast');
  Balken(8,Statfeld);
  Delay(5000);
  ClrGDP;
  GoXY(0,245);
  WritelnGDP('1 = Busplatine BUS 8         2 = Netzteil');
  WritelnGDP('3 = 3 Zoll Floppy            4 = CPU - Platine CPU 85 SC');
  WritelnGDP('5 = Floppy-Controller FDC4   6 = Speicherkarte 1M 256K');
  WritelnGDP('7 = Terminalkarte Video 80F  8 = Tastatur DIN-Tast');
  GoXY(256,100);
  Torte(100,8,Statfeld);
  Delay(5000);
  SelSize(2,2);
  GoXY(100,220);
  ClrGDP;
  WriteGDP('Fahrt z.B. mit einem PKW');
  SelSize(1,1);
  KX1:= 40; {Koordinaten der ersten Kurve}
  KY1:= 25;
  VX1:= 200;
  VY1:= 100;
  SX1:= 5;
  SY1:= 5;
  MX1:= 1;
  MY1:= 40;
  KX2:= 290; {Koordinaten der ersten Kurve}
  KY2:= 25;
  VX2:= 200;
  VY2:= 100;
  SX2:= 5;
  SY2:= 5;
  MX2:= 1;
  MY2:= 120;
  Koordi(KX1,KY1,VX1,VY1,SX1,SY1,MX1,MY1,'h','km');
  Koordi(KX2,KY2,VX2,VY2,SX2,SY2,MX2,MY2,'h','km/h');
  a:= 1.2;
  Deltat:= 10;
  s:= 0;
  t:= 0;
  v:= 0;
  GoXY(60,190);
  WriteGDP('Zurueckgelegte Strecke:');
  GoXY(300,190);
  WriteGDP('Momentangeschwindigkeit:');
  SelSize(3,3);
  GoXY(165,150);
  WriteGDP('km');
  GoXY(400,150);
  WriteGDP('km/h');
  Repeat
    v:= a * Deltat * 3.6 + v;
    s:= 0.5 * v * Deltat/3600 + s; {Zurueckgelegte Strecke}
    GoXY(50,150);
    Str(s:6:2,Meldung);
    WriteGDP(Meldung);
    GoXY(290,150);
    Str(v:6:2,Meldung);
    WriteGDP(Meldung);
    KoorPunkt(KX1,KY1,VX1,VY1,t/3600,s,MX1,MY1);
    KoorPunkt(KX2,KY2,VX2,VY2,t/3600,v,MX1,MY2);
    Case Round(t) of
          10..200 : a:= 0.04;
         201..350 : a:= -0.05;
         480..600 : a:= -0.07;
         601..700 : a:= -0.05;
        701..1100 : a:= 0.008;
       1101..1300 : a:= 0.08;
       1301..1400 : a:= 0.09;
       1401..1800 : a:= -0.011;
       1801..2200 : a:= 0.015;
       2201..2700 : a:= -0.015;
       2701..3200 : a:= 0;
       3201..3700 : a:= -0.066;
    End; {Case}
    SelEraser;
    GoXY(50,150);
    Str(s:6:2,Meldung);
    WriteGDP(Meldung);
    GoXY(290,150);
    Str(v:6:2,Meldung);
    WriteGDP(Meldung);
    SelPen;
    t:= t + Deltat;
  until t>3600;
  GoXY(50,150);
  Str(s:6:2,Meldung);
  WriteGDP(Meldung);
  GoXY(290,150);
  Str(v:6:2,Meldung);
  WriteGDP(Meldung);
  SelSize(1,1);
  delay(5000);
{$I GRATEXT.PAS }
End.
