;-----------------------------------------------------------------------;
;									;
;	Copyright (c) 1992	ELZET 80 Mikrocomputer			;
;				All Rights Reserved			;
;									;
;-----------------------------------------------------------------------;
;   GDP340 - Graphics Manager						;
;-----------------------------------------------------------------------;
;   Include file for programms using GM					;
;									;
;   This file contains type definitions used in the Graphics Manager.	;
;   It should be included in any program accessing data of these types.	;
;									;
;   Notes:								;
;-----------------------------------------------------------------------;
;   Revision history:							;
;   19.03.1993	Original Version Written		Reinhard Niebur	;
;-----------------------------------------------------------------------;

;-----------------------------------------------------------------------;
;	Integral data type sizes					;
;-----------------------------------------------------------------------;
SB	.equ	1	; Size of byte type
SW	.equ	2	; Size of word (short) type
SDW	.equ	4	; Size of double word (long) type
SPTR	.equ	4	; Size of GSP pointer
SHPTR	.equ	2	; Size of host pointer

;-----------------------------------------------------------------------;
;	BITMAP structure definition					;
;									;
;	This structure contains information describing a bitmap.	;
;-----------------------------------------------------------------------;
BITMAP_ADDR	.equ	0			; PTR	 Starting address 
BITMAP_PITCH	.equ	BITMAP_ADDR+SPTR	; ushort Pitch in bits    
BITMAP_XEXT	.equ	BITMAP_PITCH+SW		; ushort X extent         
BITMAP_YEXT	.equ	BITMAP_XEXT+SW		; ushort Y extent         
BITMAP_PSIZE	.equ	BITMAP_YEXT+SW		; ushort Pixel Size       

;-----------------------------------------------------------------------;
;	CURSOR structure definition					;
;									;
;	This structure contains cursor description information.		;
;-----------------------------------------------------------------------;
CURSOR_HOT_X	  .equ	0			; short  offset from top...
CURSOR_HOT_Y	  .equ	CURSOR_HOT_X+SW		; short  ...left-hand corner
CURSOR_WIDTH	  .equ	CURSOR_HOT_Y+SW		; ushort array width
CURSOR_HEIGHT	  .equ	CURSOR_WIDTH+SW		; ushort array height
CURSOR_PITCH	  .equ	CURSOR_HEIGHT+SW	; ushort array pitch
CURSOR_COLOR	  .equ	CURSOR_PITCH+SW		; ulong  color of cursor
CURSOR_MASK_ROP	  .equ	CURSOR_COLOR+SDW	; ushort cursor mask rop
CURSOR_SHAPE_ROP  .equ	CURSOR_MASK_ROP+SW	; ushort cursor shape rop
CURSOR_MASK_COLOR .equ	CURSOR_SHAPE_ROP+SW	; ulong  mask color
CURSOR_DATA	  .equ	CURSOR_MASK_COLOR+SDW	; PTR    pointer to cursor data

;-----------------------------------------------------------------------;
;	PATTERN structure definition					;
;									;
;	This structure contains pattern description information.	;
;-----------------------------------------------------------------------;
PATTERN_WIDTH	.equ	0			; ushort width of pattern
PATTERN_HEIGHT	.equ	PATTERN_WIDTH+SW	; ushort height of pattern
PATTERN_DEPTH	.equ	PATTERN_HEIGHT+SW	; ushort depth (bits/pixel)
PATTERN_DATA	.equ	PATTERN_DEPTH+SW	; PTR    pointer to data

;-----------------------------------------------------------------------;
;	PALET structure definition					;
;									;
;	This structure contains the rgb and intensity components for a	;
;	palette entry.							;
;-----------------------------------------------------------------------;
PALET_R		.equ	0		; uchar
PALET_G		.equ	PALET_R+SB	; uchar
PALET_B		.equ	PALET_G+SB	; uchar
PALET_I		.equ	PALET_B+SB	; uchar

;-----------------------------------------------------------------------;
;	OFFSCREEN structure definitions					;
;									;
;	These structures contain definitions of offscreen work areas	;
;-----------------------------------------------------------------------;
OFFSCREEN_ADDR	.equ	0		    ; PTR    address in gsp space of an offscreen work area
OFFSCREEN_XEXT	.equ	OFFSCREEN_ADDR+SPTR ; ushort x extension in the current screen pixel size  
OFFSCREEN_YEXT	.equ	OFFSCREEN_XEXT+SW   ; ushort y extension using default screen pitch        

;-----------------------------------------------------------------------;
;	CONFIG structure definition					;
;									;
;	This structure contains the current hardware operating		;
;	configuration.							;
;-----------------------------------------------------------------------;
CONFIG_VERSION_NUMBER	 .equ	0				; ushort
CONFIG_COMM_BUFF_SIZE	 .equ	CONFIG_VERSION_NUMBER+SW	; ulong
CONFIG_SYS_FLAGS	 .equ	CONFIG_COMM_BUFF_SIZE+SDW	; ulong
CONFIG_DEVICE_REV	 .equ	CONFIG_SYS_FLAGS+SDW		; ulong
CONFIG_NUM_MODES	 .equ	CONFIG_DEVICE_REV+SDW		; ushort
CONFIG_CURRENT_MODE	 .equ	CONFIG_NUM_MODES+SW		; ushort
CONFIG_PROGRAM_MEM_START .equ	CONFIG_CURRENT_MODE+SW		; ulong
CONFIG_PROGRAM_MEM_END	 .equ	CONFIG_PROGRAM_MEM_START+SDW	; ulong
CONFIG_DISPLAY_MEM_START .equ	CONFIG_PROGRAM_MEM_END+SDW	; ulong
CONFIG_DISPLAY_MEM_END	 .equ	CONFIG_DISPLAY_MEM_START+SDW	; ulong
CONFIG_STACK_SIZE	 .equ	CONFIG_DISPLAY_MEM_END+SDW	; ulong
CONFIG_SHARE_MEM_SIZE	 .equ	CONFIG_STACK_SIZE+SDW		; ulong
CONFIG_SHARE_HOST_ADDR	 .equ	CONFIG_SHARE_MEM_SIZE+SDW	; PTR
CONFIG_SHARE_GSP_ADDR	 .equ	CONFIG_SHARE_HOST_ADDR+SHPTR	; HPTR

CONFIG_DISP_PITCH	 .equ    CONFIG_SHARE_GSP_ADDR+SPTR	; ulong  display pitch y::x
CONFIG_DISP_VRES	 .equ    CONFIG_DISP_PITCH+SDW		; ushort vertical resolution (pixels)
CONFIG_DISP_HRES	 .equ    CONFIG_DISP_VRES+SW		; ushort horizontal resolution (pixels)
CONFIG_SCREEN_WIDE	 .equ    CONFIG_DISP_HRES+SW		; short  screen width
CONFIG_SCREEN_HIGH	 .equ    CONFIG_SCREEN_WIDE+SW		; short  screen height
CONFIG_DISP_PSIZE	 .equ    CONFIG_SCREEN_HIGH+SW		; ushort pixel size
CONFIG_PIXEL_MASK	 .equ    CONFIG_DISP_PSIZE+SW		; ulong  pixel mask
CONFIG_PALET_GUN_DEPTH	 .equ    CONFIG_PIXEL_MASK+SDW		; ushort # of bits per gun
CONFIG_PALET_SIZE	 .equ    CONFIG_PALET_GUN_DEPTH+SW	; ulong  number of palette entries
CONFIG_PALET_INSET	 .equ    CONFIG_PALET_SIZE+SDW		; short  offset from frame start to data
CONFIG_NUM_PAGES	 .equ    CONFIG_PALET_INSET+SW		; ushort number of display pages
CONFIG_NUM_OFFSCRN_AREAS .equ    CONFIG_NUM_PAGES+SW		; short  number off offscreen work areas
CONFIG_WKSP_ADDR	 .equ    CONFIG_NUM_OFFSCRN_AREAS+SW	; ulong  address of offscreen workspace
CONFIG_WKSP_PITCH	 .equ    CONFIG_WKSP_ADDR+SDW		; ulong  pitch of offscreen workspace
CONFIG_VRAM_BLOCK_WRITE  .equ    CONFIG_WKSP_PITCH+SDW		; ushort vram block write capability

;-----------------------------------------------------------------------;
;	ENVIRONMENT structure definition				;
;									;
;	This structure contains the current values of environment global;
;	variables.							;
;-----------------------------------------------------------------------;
ENVIRONMENT_XYORIGIN	.equ	0				; ulong xy drawing origin
ENVIRONMENT_PENSIZE	.equ	ENVIRONMENT_XYORIGIN+SDW	; ulong pensize height::width
ENVIRONMENT_SRCBM	.equ	ENVIRONMENT_PENSIZE+SDW		; PTR   source bitmap struct
ENVIRONMENT_DSTBM	.equ	ENVIRONMENT_SRCBM+SPTR		; PTR   dest bitmap struct
ENVIRONMENT_STYLEMASK	.equ	ENVIRONMENT_DSTBM+SPTR		; ulong styled line patn mask

;-----------------------------------------------------------------------;
;	FONT structure definition					;
;									;
;	This structure defines the data associated with a font		;
;-----------------------------------------------------------------------;
FONT_MAGIC	.equ	0			; short	    font type code
FONT_LENGTH	.equ	FONT_MAGIC+SW		; long	    font size in bytes
FONT_FACENAME	.equ	FONT_LENGTH+SDW		; char [30] facename
FONT_DEFLT	.equ	FONT_FACENAME+(SB*30)	; short	    ASCII code of default character
FONT_FIRST	.equ	FONT_DEFLT+SW		; short	    ASCII code of first character
FONT_LAST	.equ	FONT_FIRST+SW		; short	    ASCII code of last character
FONT_MAXWIDE	.equ	FONT_LAST+SW		; short	    maximum character width
FONT_MAXKERN	.equ	FONT_MAXWIDE+SW		; short	    maximum character kerning amount
FONT_CHARWIDE	.equ	FONT_MAXKERN+SW		; short	    width of characters (0=proportional)
FONT_AVGWIDE	.equ	FONT_CHARWIDE+SW	; short	    average width of characters
FONT_CHARHIGH	.equ	FONT_AVGWIDE+SW		; short	    character height
FONT_ASCENT	.equ	FONT_CHARHIGH+SW	; short	    ascent (how far above base line)
FONT_DESCENT	.equ	FONT_ASCENT+SW		; short	    descent (how far below base line)
FONT_LEADING	.equ	FONT_DESCENT+SW		; short	    leading (row bottom to next row top)
FONT_ROWPITCH	.equ	FONT_LEADING+SW		; long	    bits per row of char patterns
FONT_OPATNTBL	.equ	FONT_ROWPITCH+SDW	; long	    offset to pattern table
FONT_OLOCTBL	.equ	FONT_OPATNTBL+SDW	; long	    offset to loc table
FONT_OOWTBL	.equ	FONT_OLOCTBL+SDW	; long	    offset to offset/width table

;-----------------------------------------------------------------------;
;	FONTINFO structure definition					;
;-----------------------------------------------------------------------;
FONTINFO_FACENAME .equ	0				; char [30] facename
FONTINFO_DEFLT	  .equ	FONTINFO_FACENAME+(SB*30)	; short	    ASCII code of dflt character
FONTINFO_FIRST	  .equ	FONTINFO_DEFLT+SW		; short	    ASCII code of first character
FONTINFO_LAST	  .equ	FONTINFO_FIRST+SW		; short	    ASCII code of last character
FONTINFO_MAXWIDE  .equ	FONTINFO_LAST+SW		; short	    maximum character width
FONTINFO_AVGWIDE  .equ	FONTINFO_MAXWIDE+SW		; short	    Average width of characters
FONTINFO_MAXKERN  .equ	FONTINFO_AVGWIDE+SW		; short	    Max character kerning amount
FONTINFO_CHARWIDE .equ	FONTINFO_MAXKERN+SW		; short	    Width of characters (0=proportional)
FONTINFO_CHARHIGH .equ	FONTINFO_CHARWIDE+SW		; short	    character height
FONTINFO_ASCENT	  .equ	FONTINFO_CHARHIGH+SW		; short	    ascent (how far above base line)
FONTINFO_DESCENT  .equ	FONTINFO_ASCENT+SW		; short	    descent (how far below base line)
FONTINFO_LEADING  .equ	FONTINFO_DESCENT+SW		; short	    leading (row bottom to next row top)
FONTINFO_FONTPTR  .equ	FONTINFO_LEADING+SW		; PTR	    address of font in gsp memory
FONTINFO_ID	  .equ	FONTINFO_FONTPTR+SPTR		; short	    id of font (set at install time)

;-----------------------------------------------------------------------;
;	ENVTEXT structure definition					;
;									;
;	This structure defines the data associated with the text	;
;	environment							;
;-----------------------------------------------------------------------;
ENVTEXT_INSTALLED .equ	0			; short Number of fonts installed
ENVTEXT_ALLOCATED .equ	ENVTEXT_INSTALLED+SW	; short Number of slots allocated
ENVTEXT_FONT	  .equ	ENVTEXT_ALLOCATED+SW	; PTR   Pointer to installed fonts
ENVTEXT_SELECTED  .equ	ENVTEXT_FONT+SDW	; PTR   Currently selected font address
ENVTEXT_ALIGN	  .equ	ENVTEXT_SELECTED+SDW	; short Alignment (relative to baseline or topleft)
ENVTEXT_CHAREXTRA .equ	ENVTEXT_ALIGN+SW	; short Additional intercharacter spacing
ENVTEXT_EFFECTS   .equ	ENVTEXT_CHAREXTRA+SW	; long  Special effects
ENVTEXT_XPOSN	  .equ	ENVTEXT_EFFECTS+SDW	; short Current X text position
ENVTEXT_YPOSN	  .equ	ENVTEXT_XPOSN+SW	; short Current Y text position
