/*----------------------------------------------------------------------
 *			 TMS340 Graphics Library
 *       Copyright (c) 1987-1990 Texas Instruments Incorporated.
 *                         All Rights Reserved
 *----------------------------------------------------------------------
 * Demonstration Program
 *
 *     Test the encode_rect and decode_rect functions.	Draw a picture in
 *     the upper left corner of the screen.  Capture the picture in
 *     compressed form using the encode_rect function.	Copy the picture
 *     to various positions on the lower right portion of the screen
 *     using the decode_rect function.
 *----------------------------------------------------------------------
 * Revision History:
 *   08/19/87...Original version written...................J.R. Van Aken
 *   04/18/90...Modified for general portability...........J.R. Van Aken
 *   04.02.1993	Anpassung an GDP340 GM			Reinhard Niebur
 *----------------------------------------------------------------------
 */

#include "gmtest.h"

#define  NCOLORS      9       /* number of colors used */
#define  MAXSIZE      8192    /* max bytes used to store picture */

#define  BLACK	      0       /* symbolic color names */
#define  CLOUDWHITE   1
#define  SEABLUE      2
#define  SKYBLUE      3
#define  BOATRED      4
#define  SAILWHITE    5
#define  FRAMEBROWN   6
#define  SUNYELLOW    7

/* Use these colors if display has at least 16 colors. */
static PALET palet[NCOLORS] = {
    { 0x00, 0x00, 0x00, 0x00 }, 	/*  0 = BLACK	      */
    { 0xDD, 0xEE, 0xFF, 0x88 }, 	/*  1 = CLOUDWHITE    */
    { 0x33, 0x55, 0xFF, 0x77 }, 	/*  2 = SEABLUE       */
    { 0x33, 0xCC, 0xDD, 0xAA }, 	/*  3 = SKYBLUE       */
    { 0xAA, 0x55, 0x00, 0x99 }, 	/*  4 = BOATRED       */
    { 0xEE, 0xEE, 0xEE, 0xDD }, 	/*  5 = SAILWHITE     */
    { 0xCC, 0x77, 0x33, 0x44 }, 	/*  6 = FRAMEBROWN    */
    { 0xFF, 0xCC, 0x22, 0xEE }, 	/*  7 = SUNYELLOW     */
};

/* Define relative coordinates of sailboat hull and sails. */
static POINT2D  boat[] = { {35, 179}, {102, 179}, {87, 191}, {56, 189} };
static POINT2D sail1[] = { {62, 128}, { 62, 175}, {41, 175}, { 0,   0} };
static POINT2D sail2[] = { {65, 126}, {102, 175}, {65, 175}, { 0,   0} };

void test09(void)
{
long hres, vres, i, w, h, x, y, vx, vy;
PTR picture;

    /* Initialize graphics environment. */
    if (!set_config(MODE, 1))
	error("Invalid graphics mode number");
    clear_screen(BLACK);
    set_palet(NCOLORS, 0, palet);
    get_config(&cfg);
    hres = cfg.mode.disp_hres;	 /* horizontal resolution */
    vres = cfg.mode.disp_vres;	 /* vertical resolution */

    picture = (PTR) gsp_malloc(MAXSIZE);
    if (!picture)
	error("Error gsp_malloc picture buffer");
    
    /* Draw picture in upper left corner of screen. */
    w = 68*hres/256;			   /* image width in pixels */
    h = 57*vres/256;			   /* image height in pixels */
    for (i = 0; i < 4; i++) {
	boat[i].x    = w*boat[i].x/256;    /* scale boat coordinates */
	boat[i].y    = h*boat[i].y/256;
	sail1[i].x   = w*sail1[i].x/256;
	sail1[i].y   = h*sail1[i].y/256;
	sail2[i].x   = w*sail2[i].x/256;
	sail2[i].y   = h*sail2[i].y/256;
    }
    if (cfg.mode.palet_size >= NCOLORS) {    /* if color display */
	set_fcolor(SKYBLUE);
	fill_rect((short)w, (short)(163*h/256), 0, 0);
	set_fcolor(SEABLUE);
	fill_rect((short)w, (short)(h-163*h/256), 0, (short)(163*h/256));
	set_fcolor(SUNYELLOW);
	fill_oval((short)(53*w/256), (short)(70*h/256), (short)(150*w/256), (short)(81*h/256));
    } else {				     /* if less than NCOLORS */
	set_colors(1, 0);
	set_hard_patn(0);
	patnfill_rect((short)w, (short)(163*h/256), 0, 0);
	set_hard_patn(1); 
	patnfill_rect((short)w, (short)(h-163*h/256), 0, (short)(163*h/256));
	set_hard_patn(2);
	patnfill_oval((short)(53*w/256), (short)(70*h/256), (short)(150*w/256), (short)(81*h/256));
    }
    set_fcolor(CLOUDWHITE);
    fill_oval((short)(142*w/256), (short)(28*h/256), (short)(9*w/256), (short)(63*h/256));
    fill_oval((short)(102*w/256), (short)(23*h/256), (short)(78*w/256), (short)(84*h/256));
    fill_oval((short)(27*w/256), (short)(14*h/256), (short)(197*w/256), (short)(63*h/256));
    fill_oval((short)(22*w/256), (short)(12*h/256), (short)(215*w/256), (short)(72*h/256));
    set_fcolor(BOATRED);
    fill_convex(4, boat);
    set_fcolor(SAILWHITE);
    fill_convex(3, sail1);
    fill_convex(3, sail2);
    set_fcolor(FRAMEBROWN);
    frame_rect((short)w, (short)h, 0, 0, 5, 5);
    set_fcolor(SAILWHITE);
    draw_rect((short)(w-1), (short)(h-1), 0, 0);

    /* Run encode picture. */
    encode_rect((short)w, (short)h, 0, 0, picture, MAXSIZE, 0);

    /* Copy encoded picture to various parts of screen. */
    x = w;
    y = h;
    vx = 1 + hres/256;
    vy = 1 + vres/256;
    for (;;)
    {
	set_clip_rect((short)(hres-2*w), (short)(vres-2*h), (short)w, (short)h);
	if (cpw((short)(x += vx), (short)h))
	    vx = -vx;
	if (cpw((short)w, (short)(y += vy)))
	    vy = -vy;
	set_clip_rect((short)(hres-w), (short)(vres-h), (short)w, (short)h);
	wait_scan((short)y+(short)h);
	decode_rect((short)x, (short)y, picture);
	if (kbhit())
	    break;
    }
    getch();
    gsp_free(picture);
    clear_page(BLACK);
}

