/*----------------------------------------------------------------------
 *			 TMS340 Graphics Library
 *          Copyright (c) 1990 Texas Instruments Incorporated.
 *                         All Rights Reserved
 *----------------------------------------------------------------------
 * Demonstration Program
 *
 *   Use get_modeinfo to show and set_config function to cycle through
 *   all graphics modes supported by current graphics configuration.
 *   Print mode number, screen resolution, and pixel depth to screen.
 *
 *----------------------------------------------------------------------
 *   03/27/90...Original version written..................J.R. Van Aken
 *   02.02.1993	Anpassung an GDP340 GM			Reinhard Niebur
 *----------------------------------------------------------------------
 */

#include "gmtest.h"

#define	MAXBUFS	5	/* max. number of buffers needed	*/

OFFSCREEN_AREA	offscrn[MAXBUFS];
MODEINFO modeinfo;

void test20(void)
{
short mode, x, y, n, k, nbufs;
char c[80];

    set_config(MODE, 1);
    clear_page(BLACK);

    get_fontinfo(0, &fontinfo);	 /* get system font parameters */

    x = y = 10;
    for( mode = 0; get_modeinfo(mode, &modeinfo); mode++)
    {
	sprintf(c,"Graphics mode %d  resolution = %d -by- %d",
		mode, modeinfo.disp_hres, modeinfo.disp_vres);
	text_out(x, y, c);
	y += fontinfo.charhigh;
    }
    text_out(x,y,"Press any key to step through modes (ESC to abort)");
    if(getch() == 0x1b)
    {
        clear_page(BLACK);
	return;
    }    
    for (mode = 0; set_config(mode, 1); mode++)
    {
	clear_page(BLACK);
	get_config(&cfg);
	hres = cfg.mode.disp_hres;	 /* horizontal resolution */
	vres = cfg.mode.disp_vres;	 /* vertical resolution */
	get_fontinfo(0, &fontinfo);	 /* get system font parameters */

	frame_rect(hres, vres, 0, 0, hres/32, vres/32);
	x = hres/16;
	y = vres/16;
	sprintf(c,"Graphics mode %d",mode);
	text_out(x, y, c);
	y += fontinfo.charhigh;

	sprintf(c,"Resolution = %d -by- %d",hres,vres);
	text_out(x, y, c);
	y += fontinfo.charhigh;

	sprintf(c,"Pixel depth = %d",cfg.mode.disp_psize);
	text_out(x, y, c);
	y += fontinfo.charhigh;

	if (cfg.mode.num_pages > 1)
	    text_out(x, y, "  double buffered");
	y += fontinfo.charhigh;


	if ((nbufs = cfg.mode.num_offscrn_areas) > MAXBUFS)
	    nbufs = MAXBUFS;
	get_offscreen_memory(nbufs, offscrn);
	if (!nbufs)
	    text_out(x,y, "No off-screen buffers available.");
	else
	    for (n = 0; n < nbufs; n++)
	    {
		sprintf(c,"Buffer %d:  Addr = %08lx  xext = %d  yext = %d",
			n,offscrn[n].addr, offscrn[n].xext,offscrn[n].yext);
		text_out(x,y,c);
		y += fontinfo.charhigh;
	    }
	if(getch() == 0x1b)
	    break;
	clear_page(BLACK);
    }
    clear_page(BLACK);
}
