/*----------------------------------------------------------------------
 *			 TMS340 Graphics Library
 *          Copyright (c) 1990 Texas Instruments Incorporated.
 *                         All Rights Reserved
 *----------------------------------------------------------------------
 * Demonstration Program
 *
 *   Test the draw_point function.  Draw a variety of lissajous patterns
 *   composed of dots.	The patterns are created by driving the x and y
 *   coordinates of the dots with digital sinusoidal generators running
 *   at slightly different frequencies.  The fixed-point numbers used in
 *   the sinusoid calculations are 32-bit values with 16-bit fractions.
 *----------------------------------------------------------------------
 * Revision History:
 *   03/26/90...Original version written...................J.R. Van Aken
 *   03.02.1993	Anpassung an GDP340 GM			Reinhard Niebur
 *----------------------------------------------------------------------
 */

#include "gmtest.h"

#define N	  6		/* angular increment = 1>>N radians */
#define M	  5		/* log2(# of lines in grid pattern */
#define TWOPI	  411775l	/* fixed point 2*PI */

void test05(void)
{
long h, v, x0, y0, ux, vx, uy, vy, i, j, k;
long color;

    /* Initialize graphics environment. */
    if (!set_config(MODE, 1))
	error("Invalid graphics mode number");
    clear_screen(BLACK);
    get_config(&cfg);
    hres = cfg.mode.disp_hres;	 /* horizontal resolution */
    vres = cfg.mode.disp_vres;	 /* vertical resolution */

    /* Draw various lissajous patterns of dots. */
    h = (long)hres;
    v = (long)vres;
    for (;;)
    {
	for (k = 1; k < 8; k++)
	    for (j = 2, color = -1; j != 0; j--, color++)
	    {
		x0 = (h/2 << 16) + (1 << 15);
		vx = x0 - (4 << 16);
		ux = 0;
		y0 = (v/2 << 16) + (1 << 15);
		vy = y0 - (4 << 16);
		uy = 0;
		set_fcolor(color);
		for (i = (TWOPI<<N)>>16+1-k; i >= 0; i--)
		{
		    x = (short)((x0 + vx) >> 16);
		    y = (short)((y0 + vy) >> 16);
		    draw_point(x, y);
		    ux -= vx >> N;
		    vx += ux >> N;
		    uy -= (vy >> N) + (vy >> N+k);
		    vy += (uy >> N) + (uy >> N+k);
		}
		for (i = 30; i != 0; i--)
		    wait_scan((short)v);
/* delay(30);  tut nicht warum !!!!! */
		if (kbhit())
		{
		    getch();
		    clear_page(BLACK);
		    return;
		}
	    }
    }
}

