/*----------------------------------------------------------------------
 *			 TMS340 Graphics Library
 *       Copyright (c) 1987-1990 Texas Instruments Incorporated.
 *                         All Rights Reserved
 *----------------------------------------------------------------------
 * Demonstration Program
 *
 *   Test seed fill routines.  This program tests the seed_fill and
 *   seed_patnfill functions.  It draws a spiral shape on the screen,
 *   and then fills it--first with a solid color, and then with a
 *   pattern.
 *----------------------------------------------------------------------
 * Revision History:
 *   03/23/87...Original version written...................J.R. Van Aken
 *   04/12/90...Modified for general portability...........JRV
 *   05.02.1993	Anpassung an GDP340 GM			Reinhard Niebur
 *----------------------------------------------------------------------
 */

#include "gmtest.h"

void test17(void)
{
long x1, y1, x2, y2, color0, color1;
short i, j;

    /* Initialize graphics environment. */
    if (!set_config(MODE, 1))
	error("Invalid graphics mode number");
    clear_screen(0);
    get_config(&cfg);
    hres = cfg.mode.disp_hres;	 /* horizontal resolution */
    vres = cfg.mode.disp_vres;	 /* vertical resolution */

    /* Infinite loop below seed fills areas on screen. */
    set_draw_origin(hres/2, vres/2);
    set_pensize(1+hres/256, 1+vres/256);
    color0 = 3;
    color1 = 8;
    patn = 0;
    for (j = 0; ; j++) {
	/* Clear screen and draw spiral pattern in gray. */
	clear_screen(0);
	set_fcolor(15);
	x1 = (long)hres/16 << 16;
	y1 = (long)vres/16 << 16;
	while (!cpw((short)(x1>>16), (short)(y1>>16))) {
	    x2 = x1;
	    y2 = y1;
	    if (j & 1) {
		x1 -= y2 >> 3;
		y1 += x2 >> 4;
	    } else {
		y1 -= x2 >> 3;
		x1 += y2 >> 4;
	    }
	    pen_line((short)(x1>>16), (short)(y1>>16),
		     (short)(x2>>16), (short)(y2>>16));
	}

	/* Seed fill connected region with solid color. */
	--color1;
	set_fcolor(color1);
	seed_fill(0, 0);
	delay(30);	   /* 0.5-second delay */

	/* Seed fill connected region with two-color pattern. */
	while (++color0 == color1)
	    ;
	if (color0 == 14)
	    color0 = 1;
	set_bcolor(color0);
	while (--color1 == color0)
	    ;
	if (color1 <= 2)
	    color1 = 14;
	set_fcolor(color1);
	set_hard_patn(patn);
	if (++patn == NUM_PATNS)
	    patn = 0;
	seed_patnfill(0, 0);
	delay(60);	   /* 1-second delay */
	if (kbhit())
	   break;
    }
    getch();
    clear_page(BLACK);
}

