/*----------------------------------------------------------------------*/
/*									*/
/*	Copyright (c) 1992	ELZET 80 Mikrocomputer			*/
/*				All Rights Reserved			*/
/*									*/
/*----------------------------------------------------------------------*/
/*   GDP340 - Graphics Manager						*/
/*----------------------------------------------------------------------*/
/*   Function library host side						*/
/*									*/
/*   Handle GDP340 GM in an IBM PC or compatible with PC-ECB-Bus Adapter*/
/*   or IBM PC compatible ECB-Bus board using I/O mapped communication	*/
/*									*/
/*   Notes:								*/
/*----------------------------------------------------------------------*/
/*   Revision history:							*/
/*   22.07.1992	Original Version Written		Reinhard Niebur	*/
/*   24.02.1993	New functions added			Reinhard Niebur	*/
/*   07.04.1993	Version 2.0c				Reinhard Niebur	*/
/*----------------------------------------------------------------------*/

/* PC Host port addresses */
#define	haddhh	0x027C	/* Address high word */
#define	haddhl	0x027D
#define	haddlh	0x027E	/* Address low word */
#define	haddll	0x027F
#define	hctrlh	0x027A	/* Host control register */
#define	hctrll	0x027B
#define	hdatah	0x0278	/* Host data register */
#define	hdatal	0x0279

#include <conio.h>

/* local definitions */

static ulong cmdbuf;
static ulong argbuf;

static void put_hdata(register ushort data)
{
    outp(hdatal,data & 0x00ff);
    outp(hdatah,(data & 0xff00) >> 8);
}

static ushort get_hdata(void)
{
register ushort data;
    data = inp(hdatal) & 0x00ff;
    data += (inp(hdatah) & 0x00ff) << 8;
    return(data);
}

static void put_hctrl(register ushort data)
{
    outp(hctrll,data & 0x00ff);
    outp(hctrlh,(data & 0xff00) >> 8);
}

static ushort get_hctrl(void)
{
register ushort data;
    data = inp(hctrll) & 0x00ff;
    data += (inp(hctrlh) & 0x00ff) << 8;
    return(data);
}

static ushort get_haddl(void)
{
register ushort data;
    data  = ((ushort)(inp(haddlh) & 0x00ff)) <<  8;
    data += ((ushort)(inp(haddll) & 0x00ff));
    return(data);
}

static ulong get_hadd(void)
{
register ulong data;
    data  = ((unsigned long)(inp(haddhh) & 0x00ff)) << 24;
    data += ((unsigned long)(inp(haddhl) & 0x00ff)) << 16;
    data += ((unsigned long)(inp(haddlh) & 0x00ff)) <<  8;
    data += ((unsigned long)(inp(haddll) & 0x00ff));
    return(data);
}

static void put_hadd(register ulong addr)
{
    outp(haddll,(short) (addr & 0x000000ffl));
    outp(haddlh,(short)((addr & 0x0000ff00l) >>  8));
    outp(haddhl,(short)((addr & 0x00ff0000l) >> 16));
    outp(haddhh,(short)((addr & 0xff000000l) >> 24));
}

/* some macros for better porting */

#define	c2d(c)		put_hdata((ushort)(c))
#define	uc2d(uc)	put_hdata((ushort)(uc))
#define	s2d(s)		put_hdata((ushort)(s))
#define	us2d(us)	put_hdata((ushort)(us))
#define	l2d(l)		put_hdata((ushort)((l) & 0x0000FFFFl)); \
			put_hdata((ushort)(((l) & 0xFFFF0000l) >> 16))
#define	ul2d(ul)	put_hdata((ushort)((ul) & 0x0000FFFFl)); \
			put_hdata((ushort)(((ul) & 0xFFFF0000l) >> 16))
#define	ret_s		((short)get_haddl())
#define	ret_us		((ushort)get_haddl())
#define	ret_l		((long)get_hadd())
#define	ret_ul		((ulong)get_hadd())
#define	ret_PTR		((PTR)get_hadd())
#define	haddr(addr)	put_hadd((ulong)(addr))

static void gm_cmd(ushort cmd)
{
/* If last command doesn't return any value, wait until GDP is ready */
    while ((get_hctrl() & MSGOUT) != MSGOUT0);
/* Set command buffer address */
    haddr(cmdbuf);
/* Put command */
    put_hdata(cmd);
}

static void gm_sync_cmd(short wait)
{
/* Send message 1 */
    put_hctrl((get_hctrl() & ~MSGIN) | MSGIN1);
/* Wait until GDP send Message 1 */
    while ((get_hctrl() & MSGOUT) != MSGOUT1);
/* Send message 0 */
    put_hctrl(get_hctrl() & ~MSGIN);
    if (!wait)	/* Command is a function that doesn't return */
	return;	/* any value, so we must not ... */
/* Wait until GDP send Message 0 */
    while ((get_hctrl() & MSGOUT) != MSGOUT0);
}

/* Peek 34010 memory indirectly through the host port */
static ushort hi_peek(ulong address)
{
    outp(haddll,(short)(address & 0x000000ffl));
    outp(haddlh,(short)((address & 0x0000ff00l) >>  8));
    outp(haddhl,(short)((address & 0x00ff0000l) >> 16));
    outp(haddhh,(short)((address & 0xff000000l) >> 24));
    return(get_hdata());
}

/* Poke 34010 memory indirectly through the host port */
hi_poke(ulong address, ushort data)
{
    outp(haddll,(short)(address & 0x000000ffl));
    outp(haddlh,(short)((address & 0x0000ff00l) >>  8));
    outp(haddhl,(short)((address & 0x00ff0000l) >> 16));
    outp(haddhh,(short)((address & 0xff000000l) >> 24));
    put_hdata(data);
}

short gm_init(void)
{
ushort data;
#define DLY 500000
ulong dly;
ulong pc;
#ifndef DOWNLOAD
    hi_poke(CNTL_ADR,0x8c00);	/* Frameload, 25.175 MHz */ 
/* initialize host control register */
    put_hctrl((unsigned short)(HLT | CF | INCR | INCW | NMIMODE));
/* Get start adr of code from vector table */
    pc = (unsigned long)hi_peek(GDP_RES_VECT);
    pc += ((unsigned long)hi_peek(GDP_RES_VECT+16)) << 16;
/* Point NMI vector to start of code */
    hi_poke(NMI_LOW,(unsigned short)(pc & 0x0000FFFFl));
    hi_poke(NMI_HIGH,(unsigned short)((pc & 0xFFFF0000l) >> 16));
    data = get_hctrl();		/* read host control register */
    data &= ~HLT;		/* reset the halt bit */
    data = data | NMI | NMIMODE;  /* set NMI and NMI Mode (no stack usage) */
    data &= ~CF;		/* Turn Cache on */
    put_hctrl(data);		/* let 'er rip */
#endif
/* Synchronize host and gsp communication */
    data = get_hctrl() & ~MSGIN;
    data |= MSGIN7;
    put_hctrl(data);		/* Send message 7 */
    dly = DLY;			/* Wait until GDP send Message 7 */
    while (((get_hctrl() & MSGOUT) != MSGOUT7) && --dly);
    if (!dly) return 0;
    put_hctrl(get_hctrl() & ~MSGIN);		/* Send message 0 */
    dly = DLY;			/* Wait until GDP send Message 0 */
    while (((get_hctrl() & MSGOUT) != MSGOUT0) && --dly);
    if (!dly) return 0;
/* Get address of GM communication buffer */
    cmdbuf  = ((unsigned long)(inp(haddhh) & 0x00ff)) << 24;
    cmdbuf += ((unsigned long)(inp(haddhl) & 0x00ff)) << 16;
    cmdbuf += ((unsigned long)(inp(haddlh) & 0x00ff)) <<  8;
    cmdbuf += ((unsigned long)(inp(haddll) & 0x00ff));
    argbuf = cmdbuf + 16;
    return 1;
}

void bitblt(short w, short h, short xs, short ys, short xd, short yd)
{
    gm_cmd(Bitblt);
    s2d(w);
    s2d(h);
    s2d(xs);
    s2d(ys);
    s2d(xd);
    s2d(yd);
    gm_sync_cmd(0);
}

void clear_frame_buffer(ulong color)
{
    gm_cmd(Clear_Frame_Buffer);
    ul2d(color);
    gm_sync_cmd(0);
}

void clear_page(ulong color)
{
    gm_cmd(Clear_Page);
    ul2d(color);
    gm_sync_cmd(0);
}

void clear_screen(ulong color)
{
    gm_cmd(Clear_Screen);
    ul2d(color);
    gm_sync_cmd(0);
}

short cpw(short x, short y)
{
    gm_cmd(Cpw);
    s2d(x);
    s2d(y);
    gm_sync_cmd(1);
    return (ret_s);
}

PTR cvxyl(short x, short y)
{
    gm_cmd(Cvxyl);
    s2d(x);
    s2d(y);
    gm_sync_cmd(1);
    return (ret_PTR);
}

short decode_rect(short xleft, short ytop, PTR buf)
{
    gm_cmd(Decode_Rect);
    s2d(xleft);
    s2d(ytop);
    ul2d(buf);
    gm_sync_cmd(1);
    return (ret_s);
}

short delete_font(short id)
{
    gm_cmd(Delete_Font);
    s2d(id);
    gm_sync_cmd(1);
    return (ret_s);
}

void draw_line(short x1, short y1, short x2, short y2)
{
    gm_cmd(Draw_Line);
    s2d(x1);
    s2d(y1);
    s2d(x2);
    s2d(y2);
    gm_sync_cmd(0);
}

void draw_oval(short w, short h, short xleft, short ytop)
{
    gm_cmd(Draw_Oval);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    gm_sync_cmd(0);
}

void draw_ovalarc(short w, short h, short xleft,
 short ytop, short theta, short arc)
{
    gm_cmd(Draw_OvalArc);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    s2d(theta);
    s2d(arc);
    gm_sync_cmd(0);
}

void draw_piearc(short w, short h, short xleft,
 short ytop, short theta, short arc)
{
    gm_cmd(Draw_PieArc);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    s2d(theta);
    s2d(arc);
    gm_sync_cmd(0);
}

void draw_point(short x, short y)
{
    gm_cmd(Draw_Point);
    s2d(x);
    s2d(y);
    gm_sync_cmd(0);
}

void draw_polyline(short n, POINT2D *vert)
{
register short i;
register short *p = (short *)vert;
    gm_cmd(Draw_PolyLine);
    s2d(n);
    for(i=0;i<n;i++)
    {
	s2d(*p++);
	s2d(*p++);
    }
    gm_sync_cmd(0);
}

void draw_rect(short w, short h, short xleft, short ytop)
{
    gm_cmd(Draw_Rect);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    gm_sync_cmd(0);
}

ulong encode_rect(short w, short h, short xleft,
 short ytop, PTR buf, ulong bufsize, ushort scheme)
{
    gm_cmd(Encode_Rect);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    ul2d(buf);
    ul2d(bufsize);
    us2d(scheme);
    gm_sync_cmd(1);
    return (ret_ul);
}

ulong field_extract(PTR gptr, ushort fs)
{
    gm_cmd(Field_Extract);
    ul2d(gptr);
    us2d(fs);
    gm_sync_cmd(1);
    return (ret_ul);
}

void field_insert(PTR gptr, short fs, ulong val)
{
    gm_cmd(Field_Insert);
    ul2d(gptr);
    s2d(fs);
    ul2d(val);
    gm_sync_cmd(0);
}

void fill_convex(short n, POINT2D *vert)
{
register short i;
register short *p = (short *)vert;
    gm_cmd(Fill_Convex);
    s2d(n);
    for(i=0;i<n;i++)
    {
	s2d(*p++);
	s2d(*p++);
    }
    gm_sync_cmd(0);
}

void fill_oval(short w, short h, short xleft, short ytop)
{
    gm_cmd(Fill_Oval);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    gm_sync_cmd(0);
}

void fill_piearc(short w, short h, short xleft, short ytop,
 short theta, short arc)
{
    gm_cmd(Fill_PieArc);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    s2d(theta);
    s2d(arc);
    gm_sync_cmd(0);
}

void fill_polygon(short n, POINT2D *vert)
{
short i;
short *p = (short *)vert;
    gm_cmd(Fill_Polygon);
    s2d(n);
    for(i=0;i<n;i++)
    {
	s2d(*p++);
	s2d(*p++);
    }
    gm_sync_cmd(0);
}

void fill_rect(short w, short h, short xleft, short ytop)
{
    gm_cmd(Fill_Rect);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    gm_sync_cmd(0);
}

void frame_oval(short w, short h, short xleft, short ytop, short dx, short dy)
{
    gm_cmd(Frame_Oval);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    s2d(dx);
    s2d(dy);
    gm_sync_cmd(0);
}

void frame_rect(short w, short h, short xleft, short ytop, short dx, short dy)
{
    gm_cmd(Frame_Rect);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    s2d(dx);
    s2d(dy);
    gm_sync_cmd(0);
}

void get_colors(ulong *fcol, ulong *bcol)
{
ulong p;
    gm_cmd(Get_Colors);
    gm_sync_cmd(1);
    haddr(argbuf);
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    *fcol = p;
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    *bcol = p;
}

void get_config(CONFIG *config)
{
short i;
CONFIG temp;
ushort *p = (ushort *)&temp;
    gm_cmd(Get_Config);
    gm_sync_cmd(1);
    haddr(argbuf);
    for(i=0;i<sizeof(CONFIG)/2;i++)
	*p++ = get_hdata();
    *config = temp;
}

short get_curs_state(void)
{
    gm_cmd(Get_Curs_State);
    gm_sync_cmd(1);
    return (ret_s);
}

void get_curs_xy(short *px, short *py)
{
    gm_cmd(Get_Curs_XY);
    gm_sync_cmd(1);
    haddr(argbuf);
    *px = get_hdata();
    *py = get_hdata();
}

void get_env(ENVIRONMENT *env)
{
ulong p;
    gm_cmd(Get_Env);
    gm_sync_cmd(1);
    haddr(argbuf);
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    env->xyorigin = p;
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    env->pensize = p;
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    env->srcbm = p;
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    env->dstbm = p;
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    env->stylemask = p;
}

short get_fontinfo(short id, FONTINFO *f)
{
ushort s[15];
ulong p;
register short i,ret;
    gm_cmd(Get_FontInfo);
    s2d(id);
    gm_sync_cmd(1);
    ret = ret_s;
    haddr(argbuf);
    for(i=0; i < 15; i++)
	s[i] = get_hdata();
    strcpy(f->facename,(char *)s);
    f->first     = get_hdata();
    f->last      = get_hdata();
    f->deflt     = get_hdata();
    f->maxwide   = get_hdata();
    f->avgwide   = get_hdata();
    f->maxkern   = get_hdata();
    f->charwide  = get_hdata();
    f->charhigh  = get_hdata();
    f->ascent    = get_hdata();
    f->descent   = get_hdata();
    f->leading   = get_hdata();
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    f->fontptr   = p;
    f->id        = get_hdata();
    return (ret);
}

short get_modeinfo(short index, MODEINFO *modeinfo)
{
register ulong p;
register short ret;
    gm_cmd(Get_ModeInfo);
    s2d(index);
    gm_sync_cmd(1);
    ret = ret_s;
    haddr(argbuf);
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    modeinfo->disp_pitch = p;
    modeinfo->disp_vres = get_hdata();
    modeinfo->disp_hres = get_hdata();
    modeinfo->screen_wide = get_hdata();
    modeinfo->screen_high = get_hdata();
    modeinfo->disp_psize = get_hdata();
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    modeinfo->pixel_mask = get_hdata();
    modeinfo->palet_gun_depth = get_hdata();
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    modeinfo->palet_size = get_hdata();
    modeinfo->palet_inset = get_hdata();
    modeinfo->num_pages = get_hdata();
    modeinfo->num_offscrn_areas = get_hdata();
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    modeinfo->wksp_addr = p;
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    modeinfo->wksp_pitch = p;
    modeinfo->silicon_capability = get_hdata();
    return (ret);
}

ulong get_nearest_color(uchar r, uchar g, uchar b, uchar i)
{
    gm_cmd(Get_Nearest_Color);
    uc2d(r);
    uc2d(g);
    uc2d(b);
    uc2d(i);
    gm_sync_cmd(1);
    return (ret_ul);
}

void get_offscreen_memory(short num_blocks,OFFSCREEN_AREA *offscreen)
{
short i;
ulong p;
    gm_cmd(Get_Offscreen_Memory);
    s2d(num_blocks);
    gm_sync_cmd(1);
    haddr(argbuf);
    for(i=0;i<num_blocks;i++)
    {
	p = (ulong)get_hdata();
	p += ((ulong)get_hdata())<<16;
	offscreen[i].addr = p;
	offscreen[i].xext = get_hdata();
	offscreen[i].yext = get_hdata();
    }
}

void get_palet(short palet_size, PALET *palet)
{
short i;
ushort c;
    gm_cmd(Get_Palet);
    s2d(palet_size);
    gm_sync_cmd(1);
    haddr(argbuf);
    for(i=0;i<palet_size;i++)
    {
	c = get_hdata();
	palet[i].g = (uchar)((c & 0xFF00) >> 8);
	palet[i].r = (uchar)(c & 0x00FF);
	c = get_hdata();
	palet[i].i = (uchar)((c & 0xFF00) >> 8);
	palet[i].b = (uchar)(c & 0x00FF);
    }
}

short get_palet_entry(long index, uchar *r,
 uchar *g, uchar *b, uchar *i)
{
short ret;
ushort c;
    gm_cmd(Get_Palet_Entry);
    l2d(index);
    gm_sync_cmd(1);
    ret = ret_s;
    haddr(argbuf);
    c = get_hdata();
    *g = (uchar)((c & 0xFF00) >> 8);
    *r = (uchar)(c & 0x00FF);
    c = get_hdata();
    *i = (uchar)((c & 0xFF00) >> 8);
    *b = (uchar)(c & 0x00FF);
    return(ret);
}

ulong get_pixel(short x, short y)
{
    gm_cmd(Get_Pixel);
    s2d(x);
    s2d(y);
    gm_sync_cmd(1);
    return (ret_ul);
}

ulong get_pmask(void)
{
    gm_cmd(Get_PMask);
    gm_sync_cmd(1);
    return (ret_ul);
}

ushort get_ppop(void)
{
    gm_cmd(Get_PPop);
    gm_sync_cmd(1);
    return (ret_us);
}

short get_textattr(char *pcontrol, short count, short *val)
{
register ushort *p;
register short i, ret;
PTR c = argbuf + 32l + 16l + 32l + (long)count * 16l;
PTR v = argbuf + 32l + 16l + 32l; 
    gm_cmd(Get_TextAttr);
    ul2d(c);
    s2d(count);
    ul2d(v);
    for(i=0; i < count; i++)	/* Dummy to inc. haddr */
	s2d(0);
    p = (ushort *)pcontrol;
    for(i=0; i <= strlen(pcontrol)/2; i++)
	s2d(*p++);
    gm_sync_cmd(1);
    ret = ret_s;
    haddr(v);
    p = (ushort *)val;
    for(i=0; i < count; i++)
	*p++ = get_hdata();
    return(ret);
}

void get_text_xy(short *x, short *y)
{
    gm_cmd(Get_Text_XY);
    gm_sync_cmd(1);
    haddr(argbuf);
    *x = get_hdata();
    *y = get_hdata();
}

short get_transp(void)
{
    gm_cmd(Get_Transp);
    gm_sync_cmd(1);
    return (ret_s);
}

PTR get_vector(short trapnum)
{
    gm_cmd(Get_Vector);
    s2d(trapnum);
    gm_sync_cmd(1);
    return (ret_PTR);
}

short get_windowing(void)
{
    gm_cmd(Get_Windowing);
    gm_sync_cmd(1);
    return (ret_s);
}

void gsp2gsp(PTR src, PTR dst, ulong length)
{
    gm_cmd(Gsp2Gsp);
    ul2d(src);
    ul2d(dst);
    ul2d(length);
    gm_sync_cmd(1);
}

short get_wksp(PTR *addr, ulong *pitch)
{
short ret;
ulong p;
    gm_cmd(Get_Wksp);
    gm_sync_cmd(1);
    ret = ret_s;
    haddr(argbuf);
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    *addr = p;
    p = (ulong)get_hdata();
    p += ((ulong)get_hdata())<<16;
    *pitch = p;
    return (ret);
}

short in_font(short start_code, short end_code)
{
    gm_cmd(In_Font);
    s2d(start_code);
    s2d(end_code);
    gm_sync_cmd(1);
    return (ret_s);
}

short install_font(PTR pfont)
{
    gm_cmd(Install_Font);
    ul2d(pfont);
    gm_sync_cmd(1);
    return (ret_s);
}

void init_palet(void)
{
    gm_cmd(Init_Palet);
    gm_sync_cmd(0);
}

void init_text(void)
{
    gm_cmd(Init_Text);
    gm_sync_cmd(0);
}

short lmo(ulong n)
{
    gm_cmd(Lmo);
    ul2d(n);
    gm_sync_cmd(1);
    return (ret_s);
}

void move_pixel(short xs, short ys, short xd, short yd)
{
    gm_cmd(Move_Pixel);
    s2d(xs);
    s2d(ys);
    s2d(xd);
    s2d(yd);
    gm_sync_cmd(0);
}

short page_busy(void)
{
    gm_cmd(Page_Busy);
    gm_sync_cmd(1);
    return (ret_s);
}

void page_flip(short disp, short draw)
{
    gm_cmd(Page_Flip);
    s2d(disp);
    s2d(draw);
    gm_sync_cmd(0);
}

void patnfill_convex(short n, POINT2D *vert)
{
register short i;
register short *p = (short *)vert;
    gm_cmd(PatnFill_Convex);
    s2d(n);
    for(i=0;i<n;i++)
    {
	s2d(*p++);
	s2d(*p++);
    }
    gm_sync_cmd(0);
}

void patnfill_oval(short w, short h, short xleft, short ytop)
{
    gm_cmd(PatnFill_Oval);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    gm_sync_cmd(0);
}

void patnfill_piearc(short w, short h, short xleft,
 short ytop, short theta, short arc)
{
    gm_cmd(PatnFill_PieArc);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    s2d(theta);
    s2d(arc);
    gm_sync_cmd(0);
}

void patnfill_polygon(short n, POINT2D *vert)
{
short i;
short *p = (short *)vert;
    gm_cmd(PatnFill_Polygon);
    s2d(n);
    for(i=0;i<n;i++)
    {
	s2d(*p++);
	s2d(*p++);
    }
    gm_sync_cmd(0);
}

void patnfill_rect(short w, short h, short xleft, short ytop)
{
    gm_cmd(PatnFill_Rect);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    gm_sync_cmd(0);
}

void patnframe_oval(short w, short h, short xleft,
 short ytop, short dx, short dy)
{
    gm_cmd(PatnFrame_Oval);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    s2d(dx);
    s2d(dy);
    gm_sync_cmd(0);
}

void patnframe_rect(short w, short h, short xleft,
 short ytop, short dx, short dy)
{
    gm_cmd(PatnFrame_Rect);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    s2d(dx);
    s2d(dy);
    gm_sync_cmd(0);
}

void patnpen_line(short x1, short y1, short x2, short y2)
{
    gm_cmd(PatnPen_Line);
    s2d(x1);
    s2d(y1);
    s2d(x2);
    s2d(y2);
    gm_sync_cmd(0);
}

void patnpen_ovalarc(short w, short h, short xleft,
 short ytop, short theta, short arc)
{
    gm_cmd(PatnPen_OvalArc);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    s2d(theta);
    s2d(arc);
    gm_sync_cmd(0);
}

void patnpen_piearc(short w, short h, short xleft,
 short ytop, short theta, short arc)
{
    gm_cmd(PatnPen_PieArc);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    s2d(theta);
    s2d(arc);
    gm_sync_cmd(0);
}

void patnpen_point(short x, short y)
{
    gm_cmd(PatnPen_Point);
    s2d(x);
    s2d(y);
    gm_sync_cmd(0);
}

void patnpen_polyline(short n, POINT2D *vert)
{
register short i;
register short *p = (short *)vert;
    gm_cmd(PatnPen_PolyLine);
    s2d(n);
    for(i=0;i<n;i++)
    {
	s2d(*p++);
	s2d(*p++);
    }
    gm_sync_cmd(0);
}

ulong peek_breg(short breg)
{
    gm_cmd(Peek_BReg);
    s2d(breg);
    gm_sync_cmd(1);
    return (ret_ul);
}

void pen_line(short x1, short y1, short x2, short y2)
{
    gm_cmd(Pen_Line);
    s2d(x1);
    s2d(y1);
    s2d(x2);
    s2d(y2);
    gm_sync_cmd(0);
}

void pen_ovalarc(short w, short h, short xleft,
 short ytop, short theta, short arc)
{
    gm_cmd(Pen_OvalArc);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    s2d(theta);
    s2d(arc);
    gm_sync_cmd(0);
}

void pen_piearc(short w, short h, short xleft,
 short ytop, short theta, short arc)
{
    gm_cmd(Pen_PieArc);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    s2d(theta);
    s2d(arc);
    gm_sync_cmd(0);
}

void pen_point(short x, short y)
{
    gm_cmd(Pen_Point);
    s2d(x);
    s2d(y);
    gm_sync_cmd(0);
}

void pen_polyline(short n, POINT2D *vert)
{
register short i;
register short *p = (short *)vert;
    gm_cmd(Pen_PolyLine);
    s2d(n);
    for(i=0;i<n;i++)
    {
	s2d(*p++);
	s2d(*p++);
    }
    gm_sync_cmd(0);
}

void poke_breg(short breg, ulong val)
{
    gm_cmd(Poke_BReg);
    s2d(breg);
    ul2d(val);
    gm_sync_cmd(0);
}

void put_pixel(ulong val, short x, short y)
{
    gm_cmd(Put_Pixel);
    ul2d(val);
    s2d(x);
    s2d(y);
    gm_sync_cmd(0);
}

short rmo(ulong n)
{
    gm_cmd(Rmo);
    ul2d(n);
    gm_sync_cmd(1);
    return (ret_s);
}

void seed_fill(short x, short y
 /*, char *buf, short maxbytes*/)
{
    gm_cmd(Seed_Fill);
    s2d(x);
    s2d(y);
    gm_sync_cmd(0);
}

void seed_patnfill(short x, short y
 /*, char *buf, short maxbytes*/)
{
    gm_cmd(Seed_PatnFill);
    s2d(x);
    s2d(y);
    gm_sync_cmd(0);
}

short select_font(short id)
{
    gm_cmd(Select_Font);
    s2d(id);
    gm_sync_cmd(1);
    return (ret_s);
}

void set_bcolor(ulong color)
{
    gm_cmd(Set_BColor);
    ul2d(color);
    gm_sync_cmd(0);
}

void set_clip_rect(ushort w, ushort h, short xleft, short ytop)
{
    gm_cmd(Set_Clip_Rect);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    gm_sync_cmd(0);
}

void set_colors(ulong fcolor, ulong bcolor)
{
    gm_cmd(Set_Colors);
    ul2d(fcolor);
    ul2d(bcolor);
    gm_sync_cmd(0);
}

short set_config(short graphics_mode, short init_draw)
{
    gm_cmd(Set_Config);
    s2d(graphics_mode);
    s2d(init_draw);
    gm_sync_cmd(1);
    return (ret_s);
}

void set_curs_shape(PTR shape)
{
    gm_cmd(Set_Curs_Shape);
    ul2d(shape);
    gm_sync_cmd(0);
}

void set_curs_state(short enable)
{
    gm_cmd(Set_Curs_State);
    s2d(enable);
    gm_sync_cmd(0);
}

void set_curs_xy(short x, short y)
{
    gm_cmd(Set_Curs_XY);
    s2d(x);
    s2d(y);
    gm_sync_cmd(0);
}

void set_cursattr(ulong shape_c, ulong mask_c,
 ushort shape_a, ushort mask_a)
{
    gm_cmd(Set_CursAttr);
    ul2d(shape_c);
    ul2d(mask_c);
    us2d(shape_a);
    us2d(mask_a);
    gm_sync_cmd(0);
}

void set_draw_origin(short x, short y)
{
    gm_cmd(Set_Draw_Origin);
    s2d(x);
    s2d(y);
    gm_sync_cmd(0);
}

void set_dstbm(PTR baseaddr, short pitch,
 short xext, short yext, short psize)
{
    gm_cmd(Set_DstBm);
    ul2d(baseaddr);
    s2d(pitch);
    s2d(xext);
    s2d(yext);
    s2d(psize);
    gm_sync_cmd(0);
}

void set_fcolor(ulong color)
{
    gm_cmd(Set_FColor);
    ul2d(color);
    gm_sync_cmd(0);
}

void set_palet(long count, long index, PALET *palet)
{
register ulong *p = (ulong *)palet;
register long i;
register ushort col;
    gm_cmd(Set_Palet);
    l2d(count);
    l2d(index);
    for(i=0; i < count; i++,p++)
    {
	col = (ushort)((palet[i].g & 0x00ff) << 8);
	col |= (ushort)(palet[i].r & 0x00ff);
	us2d(col);
	col = (ushort)((palet[i].i & 0x00ff) << 8);
	col |= (ushort)(palet[i].b & 0x00ff);
	us2d(col);
    }
    gm_sync_cmd(0);
}

short set_palet_entry(long index, uchar r, uchar g, uchar b, uchar i)
{
    gm_cmd(Set_Palet_Entry);
    l2d(index);
    uc2d(r);
    uc2d(g);
    uc2d(b);
    uc2d(i);
    gm_sync_cmd(1);
    return (ret_s);
}

void set_patn(PATTERN *ppatn)
{
    gm_cmd(Set_Patn);
    us2d(ppatn->width);
    us2d(ppatn->height);
    us2d(ppatn->depth);
    ul2d(ppatn->data);
    gm_sync_cmd(0);
}

void set_pensize(short x, short y)
{
    gm_cmd(Set_PenSize);
    s2d(x);
    s2d(y);
    gm_sync_cmd(0);
}

void set_pmask(ulong pmask)
{
    gm_cmd(Set_PMask);
    ul2d(pmask);
    gm_sync_cmd(0);
}

void set_ppop(short ppop)
{
    gm_cmd(Set_PPop);
    s2d(ppop);
    gm_sync_cmd(0);
}

void set_srcbm(PTR baseaddr, short pitch,
 short xext, short yext, short psize)
{
    gm_cmd(Set_SrcBm);
    ul2d(baseaddr);
    s2d(pitch);
    s2d(xext);
    s2d(yext);
    s2d(psize);
    gm_sync_cmd(0);
}

void set_text_xy(short x, short y)
{
    gm_cmd(Set_Text_XY);
    s2d(x);
    s2d(y);
    gm_sync_cmd(0);
}

short set_textattr(char *pcontrol, short count, short *val)
{
register ushort *p = (ushort *)pcontrol;
register short i;
PTR c = argbuf + 32l + 16l + 32l + (long)count * 16l;
PTR v = argbuf + 32l + 16l + 32l; 
    gm_cmd(Set_TextAttr);
    ul2d(c);
    s2d(count);
    ul2d(v);
    for(i=0; i < count; i++)
	s2d(*val++);
    for(i=0; i <= strlen(pcontrol)/2; i++)
	s2d(*p++);
    gm_sync_cmd(1);
    return (ret_s);
}

PTR set_vector(short trapnum, PTR gptr)
{
    gm_cmd(Set_Vector);
    s2d(trapnum);
    ul2d(gptr);
    gm_sync_cmd(1);
    return (ret_PTR);
}

void set_windowing(short mode)
{
    gm_cmd(Set_Windowing);
    s2d(mode);
    gm_sync_cmd(0);
}

void set_wksp(PTR addr, ulong pitch)
{
    gm_cmd(Set_Wksp);
    ul2d(addr);
    ul2d(pitch);
    gm_sync_cmd(0);
}

void styled_line(short x1, short y1, short x2,
 short y2, long style, short mode)
{
    gm_cmd(Styled_Line);
    s2d(x1);
    s2d(y1);
    s2d(x2);
    s2d(y2);
    l2d(style);
    s2d(mode);
    gm_sync_cmd(0);
}

void styled_oval(short w, short h, short xleft,
 short ytop, long style, short mode)
{
    gm_cmd(Styled_Oval);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    l2d(style);
    s2d(mode);
    gm_sync_cmd(0);
}

void styled_ovalarc(short w, short h, short xleft,
 short ytop, short theta, short arc, long style, short mode)
{
    gm_cmd(Styled_OvalArc);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    s2d(theta);
    s2d(arc);
    l2d(style);
    s2d(mode);
    gm_sync_cmd(0);
}

void styled_piearc(short w, short h, short xleft,
 short ytop, short theta, short arc, long style, short mode)
{
    gm_cmd(Styled_PieArc);
    s2d(w);
    s2d(h);
    s2d(xleft);
    s2d(ytop);
    s2d(theta);
    s2d(arc);
    l2d(style);
    s2d(mode);
    gm_sync_cmd(0);
}

void swap_bm(void)
{
    gm_cmd(Swap_Bm);
    gm_sync_cmd(0);
}

short text_out(short x, short y, char *s)
{
register ushort *p = (ushort *)s;
register short i;
    gm_cmd(Text_Out);
    s2d(x);
    s2d(y);
    for(i=0; i <= strlen(s)/2; i++)
	us2d(*p++);
    gm_sync_cmd(1);
    return (ret_s);
}

short text_outp(char *s)
{
register ushort *p = (ushort *)s;
register short i;
    gm_cmd(Text_Outp);
    for(i=0; i <= strlen(s)/2; i++)
	us2d(*p++);
    gm_sync_cmd(1);
    return (ret_s);
}

short text_width(char *s)
{
register ushort *p = (ushort *)s;
register short i;
    gm_cmd(Text_Width);
    for(i=0; i <= strlen(s)/2; i++)
	us2d(*p++);
    gm_sync_cmd(1);
    return (ret_s);
}

void transp_off(void)
{
    gm_cmd(Transp_Off);
    gm_sync_cmd(0);
}

void transp_on(void)
{
    gm_cmd(Transp_On);
    gm_sync_cmd(0);
}

void wait_scan(short line)
{
    gm_cmd(Wait_Scan);
    s2d(line);
    gm_sync_cmd(0);
}

void zoom_rect(short ws, short hs, short xs, short ys,
 short wd, short hd, short xd, short yd, PTR rowbuf)
{
    gm_cmd(Zoom_Rect);
    s2d(ws);
    s2d(hs);
    s2d(xs);
    s2d(ys);
    s2d(wd);
    s2d(hd);
    s2d(xd);
    s2d(yd);
    ul2d(rowbuf);
    gm_sync_cmd(0);
}

PTR gsp_calloc(ulong num, ulong size)
{
    gm_cmd(Gsp_CAlloc);
    ul2d(num);
    ul2d(size);
    gm_sync_cmd(1);
    return (ret_PTR);
}

void gsp_free(PTR ptr)
{
    gm_cmd(Gsp_Free);
    ul2d(ptr);
    gm_sync_cmd(0);
}

PTR gsp_malloc(ulong size)
{
    gm_cmd(Gsp_MAlloc);
    ul2d(size);
    gm_sync_cmd(1);
    return (ret_PTR);
}

void gsp_minit(void)
{
    gm_cmd(Gsp_MInit);
    gm_sync_cmd(0);
}

PTR gsp_realloc(PTR ptr, ulong size)
{
    gm_cmd(Gsp_ReAlloc);
    ul2d(ptr);
    ul2d(size);
    gm_sync_cmd(1);
    return (ret_PTR);
}

short install_hard_font(short n)
{
    gm_cmd(Install_Hard_Font);
    s2d(n);
    gm_sync_cmd(1);
    return (ret_s);
}

void set_hard_patn(short patn_nr)
{
    gm_cmd(Set_Hard_Patn);
    s2d(patn_nr);
    gm_sync_cmd(0);
}

void set_hard_curs_shape(short id)
{
    gm_cmd(Set_Hard_Curs_Shape);
    s2d(id);
    gm_sync_cmd(0);
}

void delay(short screens)
{
    gm_cmd(Delay);
    s2d(screens);
    gm_sync_cmd(0);
}

/* SWIZZLE not implemented now */
void host2gsp(HPTR hptr, PTR gptr, ushort len, short swizzle)
{
register ushort i;
register ushort l = len / 2;
register ushort *p = (ushort *)hptr;
    while ((get_hctrl() & MSGOUT) != MSGOUT0);	/* Wait until GDP is ready */
    haddr(gptr);
    for (i=0; i<l; i++)
	us2d(*p++);
}

/* SWIZZLE not implemented now */
void gsp2host(PTR gptr, HPTR hptr, ushort len, short swizzle)
{
register ushort i;
register ushort l = len / 2;
register ushort *p = (ushort *)hptr;
    while ((get_hctrl() & MSGOUT) != MSGOUT0);	/* Wait until GDP is ready */
    haddr(gptr);
    for (i=0; i<l; i++)
	*p++ = get_hdata();
}

