/*----------------------------------------------------------------------
 *			 TMS340 Graphics Library
 *          Copyright (c) 1990 Texas Instruments Incorporated.
 *                         All Rights Reserved
 *----------------------------------------------------------------------
 * Demonstration Program
 *
 *   Display the 16 colors of the default palette.  Fill 16 horizontal
 *   bars with the 16 default colors, and label each bar with the symbol
 *   name of the color, as defined in the "colors.h" include file.
 *
 *----------------------------------------------------------------------
 * Revision History:
 *   04/10/90...Original version written...................J.R. Van Aken
 *   02.02.1993	Anpassung an GDP340 GM			Reinhard Niebur
 *----------------------------------------------------------------------
 */

#include "gmtest.h"

CONFIG cfg;
FONTINFO fontinfo;
short hres,vres,x,y;
short patn,penw,penh;
char s[129];

#define NCOLORS 16

static char *str[NCOLORS] = {
    "BLACK        ", "BLUE         ", "GREEN        ", "CYAN         ",
    "RED          ", "MAGENTA      ", "BROWN        ", "LIGHT_GRAY   ",
    "DARK_GRAY    ", "LIGHT_BLUE   ", "LIGHT_GREEN  ", "LIGHT_CYAN   ",
    "LIGHT_RED    ", "LIGHT_MAGENTA", "YELLOW       ", "WHITE        ",
};

static PALET palet1[NCOLORS], palet2[NCOLORS];

/* Print error message and exit. */
void error(char *str)
{
    printf("[ Error: ");
    if (!*str)
        printf("Unexpected EOF");
    else
        printf("%s",str);
    printf(" ]\n");
    exit(1);
}

void main(void)
{
ulong fcolor, bcolor;
short w, h, len, color;

    if (!gm_init())
        error("No Response from GDP");

    /* Initialize graphics environment. */
    if (!set_config(MODE, 1))
        error("Invalid graphics mode number");
    clear_screen(-1);
    get_config(&cfg);
    hres = cfg.mode.disp_hres;	 /* horizontal resolution */
    vres = cfg.mode.disp_vres;	 /* vertical resolution */
    len = text_width(str[0]);
    w = hres - 24 - len;
    h = vres/NCOLORS - 4;

    get_colors(&fcolor, &bcolor);
    for (color = 0; color < NCOLORS; color++) {
	set_fcolor(color);
	fill_rect(w, h, 4, 2+color*(h+4));
	set_fcolor(fcolor);
	text_out(16+w, 2+color*(h+4), str[color]);
    }

    get_palet(NCOLORS, palet1);
    set_palet(NCOLORS, 0, palet1);
    for (color = 0; color < NCOLORS; color++) {
	set_fcolor(color);
	fill_rect(w/4, h, 4+w/4, 2+color*(h+4));
	set_fcolor(fcolor);
    }

    for (color = 0; color < NCOLORS; color++)
	get_palet_entry((long)color,
		&palet2[color].r, &palet2[color].g,
		&palet2[color].b, &palet2[color].i);
    set_palet(NCOLORS, 0, palet2);
    for (color = 0; color < NCOLORS; color++) {
	set_fcolor(color);
	fill_rect(w/4, h, 4+2*w/4, 2+color*(h+4));
	set_fcolor(fcolor);
    }

    for (color = 0; color < NCOLORS; color++)
	set_palet_entry((long)color,
		palet2[color].r, palet2[color].g,
		palet2[color].b, palet2[color].i);
    get_palet(NCOLORS, palet2);
    for (color = 0; color < NCOLORS; color++) {
	set_fcolor(color);
	fill_rect(w/4, h, 4+3*w/4, 2+color*(h+4));
	set_fcolor(fcolor);
	draw_rect(w, h, 4, 2+color*(h+4));
    }
    getch();
    clear_page(BLACK);
}
