/*----------------------------------------------------------------------
 *			 TMS340 Graphics Library
 *          Copyright (c) 1990 Texas Instruments Incorporated.
 *                         All Rights Reserved
 *----------------------------------------------------------------------
 * Demonstration Program
 *
 *   Draw text to the screen using one of the GM predefined fonts, which is
 *   selected to be compatible with the resolution of the screen.
 *   Call the set_textattr function (see last statement of program) once
 *   per frame to vary the intercharacter spacing.  The effect is that
 *   the text seems to stretch and contract on the screen.
 *----------------------------------------------------------------------
 *   04/12/90...Original version written..................J.R. Van Aken
 *   02.02.1993	Anpassung an GDP340 GM			Reinhard Niebur
 *----------------------------------------------------------------------
 */

#include "gmtest.h"

CONFIG cfg;
FONTINFO fontinfo;
short hres,vres,x,y;
short patn,penw,penh;
char s[129];

#define NLINES	  3	  /* number of lines of text */
static char *str[NLINES] = { "Variable", "Intercharacter", "Spacing", };
static char wstr[80];

/* Print error message and exit. */
void error(char *str)
{
    printf("[ Error: ");
    if (!*str)
        printf("Unexpected EOF");
    else
        printf("%s",str);
    printf(" ]\n");
    exit(1);
}

void main(void)
{
short delta, i, index, wmax, w, h, yy;
short spacing;

    if (!gm_init())
        error("No Response from GDP");

    /* Initialize graphics environment. */
    if (!set_config(MODE, 1))
        error("Invalid graphics mode number");
    clear_screen(0);
    get_config(&cfg);
    hres = cfg.mode.disp_hres;	 /* horizontal resolution */
    vres = cfg.mode.disp_vres;	 /* vertical resolution */

    if (hres <= 320)
	index = install_hard_font(HARD_CORPUS16);
    else if (hres <= 640)
	index = install_hard_font(HARD_CORPUS26);
    else
	index = install_hard_font(HARD_CORPUS29);

    select_font(index);
    get_fontinfo(-1, &fontinfo);  /* get font parameters */

    /* Loop below displays examples of both
     * positive and negative intercharacter spacing.
     */
    h = 2*fontinfo.charhigh;
    spacing = 0;
    delta = 1;
    for(;;)
    {
	/* Draw text on screen. */
	set_fcolor(WHITE);
	set_ppop(8);	    /* OR */
	wmax = 0;
	yy = y = vres/2 - (2*NLINES - 1)*h/4;
	for (i = 0; i < NLINES; i++) {
	    w = text_width(str[i]);
	    if (abs(w) > wmax)
		wmax = abs(w);
	    x = (hres - w)/2;
	    text_out(x, y, str[i]);
	    y += h;
	}
	draw_rect(hres-1, vres-1, 0, 0);
	delay(5); 
	if (kbhit()) break;
	/* Erase text. */
	set_fcolor(BLACK);
	set_ppop(0);	    /* REPLACE */
	y = yy;
	if (h < 0)
	    wait_scan(y+fontinfo.charhigh);
	else
	    wait_scan(y+5*fontinfo.charhigh);
	for (i = 0; i < NLINES; i++) {
	    w = text_width(str[i]);
	    if (abs(w) > wmax)
		wmax = abs(w);
	    x = (hres - w)/2;
	    text_out(x, y, str[i]);
	    y += h;
	}

	/* Update extra intercharacter spacing. */
	if (wmax > hres)
	    delta = -delta;
	spacing += delta;
	h += 2*delta;
	set_textattr("%*e", 1, &spacing);
    }
    if(!delete_font(index))
    {
	sprintf(wstr,"Error deleting font %s",fontinfo.facename);
	text_out(10,10,wstr);
	if(kbhit())
	    getch();
    }
    getch();
    clear_page(BLACK);
}

