/*----------------------------------------------------------------------
 *			 TMS340 Graphics Library
 *          Copyright (c) 1990 Texas Instruments Incorporated.
 *                         All Rights Reserved
 *----------------------------------------------------------------------
 * Demonstration Program
 *
 *   Test the patnfill_rect function.  Show the fill patterns available
 *   as a "stack of cards" display spread across the screen.
 *   Pattern 0 is at the top left corner of the screen.
 *   Redraw the patterns with new colors once each second.
 *----------------------------------------------------------------------
 * Revision History:
 *   03/26/90...Original version written...................J.R. Van Aken
 *   03.02.1993	Anpassung an GDP340 GM			Reinhard Niebur
 *----------------------------------------------------------------------
 */

#include "gmtest.h"

CONFIG cfg;
FONTINFO fontinfo;
short hres,vres,x,y;
short patn,penw,penh;
char s[129];

/* Print error message and exit. */
void error(char *str)
{
    printf("[ Error: ");
    if (!*str)
        printf("Unexpected EOF");
    else
        printf("%s",str);
    printf(" ]\n");
    exit(1);
}

void main(void)
{
short j, w, h, dx, dy;
ulong color0, color1;

    if (!gm_init())
        error("No Response from GDP");

     /* Initialize graphics environment. */
     if (!set_config(MODE, 1))
	error("Invalid graphics mode number");
     clear_screen(-1);
     get_config(&cfg);
     hres = cfg.mode.disp_hres;	 /* horizontal resolution */
     vres = cfg.mode.disp_vres;	 /* vertical resolution */

     /* Adjust size and displacement parameters to screen dimensions. */
     w = hres/4;
     h = vres/4;
     dx = (hres - w)/(NUM_PATNS - 1);
     dy = (vres - h)/(NUM_PATNS - 1);

     /* Loop below draws geometric primitives to screen until kbhit. */
     for (j=0;;j++) {
	x = y = 0;
	color0 = j;
	color1 = j+1;
	for (patn = 0; patn < NUM_PATNS; patn++, x += dx, y += dy)
	{
	    color0 = (color0 + 7) & 15;
	    color1 = (color1 + 3) & 15;
	    set_colors(color1, color0);
	    set_hard_patn(patn);
	    patnfill_rect(w, h, x, y);
	}
	delay(60);	  /* 1-second delay */
	if (kbhit())
	    break;
    }
    getch();
    clear_page(BLACK);
}
