/*----------------------------------------------------------------------
 *			 TMS340 Graphics Library
 *       Copyright (c) 1987-1990 Texas Instruments Incorporated.
 *                         All Rights Reserved
 *----------------------------------------------------------------------
 * Demonstration Program
 *
 *   Test pen_line and patnpen_line functions.	Draw fat lines emanating
 *   in circular pattern from center of screen.  Alternate frames of
 *   solid-colored with frames of pattern-filled lines.  Insert 1-second
 *   delay between frames.
 *----------------------------------------------------------------------
 * Revision History:
 *   07/01/87...Original version written...................J.R. Van Aken
 *   04/12/90...Modified for general portability...........J.R. Van Aken
 *   04.02.1993	Anpassung an GDP340 GM			Reinhard Niebur
 *----------------------------------------------------------------------
 */

#include "gmtest.h"

CONFIG cfg;
FONTINFO fontinfo;
long hres,vres,x,y;
short patn,penw,penh;
char s[129];

/* Print error message and exit. */
void error(char *str)
{
    printf("[ Error: ");
    if (!*str)
        printf("Unexpected EOF");
    else
        printf("%s",str);
    printf(" ]\n");
    exit(1);
}

void main(void)
{
long r, t, rmin, rmax;
long color0, color1;

    if (!gm_init())
        error("No Response from GDP");

    /* Initialize graphics environment. */
    if (!set_config(MODE, 1))
	error("Invalid graphics mode number");
    clear_screen(-1);
    get_config(&cfg);
    hres = cfg.mode.disp_hres;	 /* horizontal resolution */
    vres = cfg.mode.disp_vres;	 /* vertical resolution */
    set_draw_origin((short)hres/2, (short)vres/2);
    color0 = 0;
    color1 = -1;
    patn = 0;
    rmin = hres << 16-5;
    rmax = (hres > vres) ? vres << 16-1 : hres << 16-1 ;

    /* Loop below draws circular patterns of fat lines until kbhit. */
    for (;;)
    {
	/* Draw solid-colored fat lines. */
	set_pensize((short)(1+hres/200), (short)(1+vres/200));
	clear_screen(-1);
	for (r = rmin; r <= rmax; r += r/16)
	{
            x = 0;
	    y = -r;
	    for (t = 202; t != 0; t--)
	    {
		if (cfg.mode.disp_psize >= 4)  /* color display? */
		    set_fcolor(color1--);
		pen_line((short)(x-x/4>>16), (short)(y-y/4>>16),
			 (short)(x>>16), (short)(y>>16));
                x -= y >> 5;
                y += x >> 5;
            }
	    if (kbhit())
	    {
		getch();
		clear_page(BLACK);
		return;
	    }
	}
	delay(60);	/* 1-second delay */

	/* Draw pattern-filled fat lines. */
	set_pensize((short)(2+hres/100), (short)(2+vres/100));
	clear_screen(-1);
	for (r = rmin; r <= rmax; r += r/16)
	{
            x = 0;
	    y = -r;
	    for (t = 101; t != 0; t--)
	    {
		set_colors(color1--, color0++);
		set_hard_patn(patn);
		if (++patn == NUM_PATNS)
		    patn = 0;
		patnpen_line((short)(x-x/4>>16), (short)(y-y/4>>16),
			     (short)(x>>16), (short)(y>>16));
                x -= y >> 4;
                y += x >> 4;
            }
	    if (kbhit())
	    {
		getch();
		clear_page(BLACK);
		return;
	    }
        }
	color0++;
	color1--;
	delay(60);	/* 1-second delay */
    }
}
