/*----------------------------------------------------------------------
 *			 TMS340 Graphics Library
 *       Copyright (c) 1987-1990 Texas Instruments Incorporated.
 *                         All Rights Reserved
 *----------------------------------------------------------------------
 * Demonstration Program
 *
 *   Test the draw_oval, styled_oval, draw_ovalarc, styled_ovalarc,
 *   draw_piearc and styled_piearc functions.  Animated demo draws series
 *   of pie arcs as starting angle and extent of arc changes gradually.
 *   Dimensions of enclosing rectangle also change.  Both solid arcs and
 *   styled arcs are drawn.
 *----------------------------------------------------------------------
 * Revision History:
 *   08/19/87...Original version written...................J.R. Van Aken
 *   06/14/89...Modified for TIGA standalone library.......Al Sharp
 *   11/29/89...Modified for general portability...........JRV
 *   05.02.1993	Anpassung an GDP340 GM			Reinhard Niebur
 *----------------------------------------------------------------------
 */

#include "gmtest.h"

CONFIG cfg;
FONTINFO fontinfo;
short hres,vres;
short patn,penw,penh;
char s[129];

#define  LINESTYLE  0x0FFF0FFFL	 /* 32-bit arc line-style pattern mask */

/* Print error message and exit. */
void error(char *str)
{
    printf("[ Error: ");
    if (!*str)
        printf("Unexpected EOF");
    else
        printf("%s",str);
    printf(" ]\n");
    exit(1);
}

void main(void)
{
short disppage, drawpage, phase, draw, ts, ta, dts, dta;
long x, y, z, xmax, ymax;

    if (!gm_init())
        error("No Response from GDP");

    /* Initialize graphics environment. */
    if (!set_config(DMODE, 1))
	error("Invalid graphics mode number");
    clear_page(0);
    get_config(&cfg);
    hres = cfg.mode.disp_hres;	 /* horizontal resolution */
    vres = cfg.mode.disp_vres;	 /* vertical resolution */
    set_draw_origin(hres/2, vres/2);
    set_bcolor(RED);   /* color for 0s in line-style mask */

    /* Initialize loop parameters. */
    x = xmax = (long)(hres/2 - hres/16) << 16;
    y = ymax = (long)(vres/2 - vres/8) << 16;
    z = 0;
    ts = 90;
    ta = 360;
    dts = 1;
    dta = -2;
    draw = 0;
    phase = 0;
    disppage = 1;
    drawpage = 0;

    /* Loop below draws arc or ellipse, and enclosing rect. */
    for (;;) {
	clear_page(BLACK);
	set_fcolor(DARK_GRAY);
	frame_rect((short)(x>>16-1), (short)(y>>16-1),
		   (short)(-x>>16), (short)(-y>>16), 4, 4);

	/* Inscribe arc or ellipse inside enclosing rectangle. */
	set_fcolor(WHITE);
	switch (draw) { 	  /* 1 of 9 arc or ellipse functions */
	case 0:
	    draw_oval((short)(x>>16-1), (short)(y>>16-1),
		      (short)(-x>>16), (short)(-y>>16));
	    break;
	case 1:
	case 2:
	    styled_oval((short)(x>>16-1), (short)(y>>16-1),
			(short)(-x>>16), (short)(-y>>16), LINESTYLE, draw&1);
	    break;
	case 3:
	    draw_ovalarc((short)(x>>16-1), (short)(y>>16-1),
			 (short)(-x>>16), (short)(-y>>16), ts, ta);
	    break;
	case 4:
	case 5:
	    styled_ovalarc((short)(x>>16-1), (short)(y>>16-1),
			   (short)(-x>>16), (short)(-y>>16),
			   ts, ta, LINESTYLE, draw&1);
	    break;
	case 6:
	    draw_piearc((short)(x>>16-1), (short)(y>>16-1),
			(short)(-x>>16), (short)(-y>>16), ts, ta);
	    break;
	case 7:
	case 8:
	    styled_piearc((short)(x>>16-1), (short)(y>>16-1),
			  (short)(-x>>16), (short)(-y>>16),
			  ts, ta, LINESTYLE, draw&1);
	    break;
	}

	/* Schedule page flip to occur at bottom of screen. */
	page_flip(disppage ^= 1, drawpage ^= 1);

	/* Update arc starting angle and arc extent. */
	if (abs(ts += dts) > 359 )
	    dts = -dts;
	if (abs(ta += dta) > 385)
	    dta = -dta;

	/* Update dimensions of rectangle enclosing ellipse or arc. */
	switch (phase) {
	case 0: 	      /* rectangle shrinking horizontally */
	    z += x >> 7;
	    x -= z >> 7;
	    if (x < 0) {
		x = 0;
		z = xmax;
		ts = (180 - ts) % 360;
		ta = -ta;
		dts = -dts;
		dta = -dta;
		phase = 1;
		if (++draw > 8)
		    draw = 0;
	    }
	    break;
	case 1: 	      /* rectangle growing horizontally */
	    x += z >> 7;
	    z -= x >> 7;
	    if (z < 0) {
		x = xmax;
		z = 0;
		phase = 2;
	    }
	    break;
	case 2: 	      /* rectangle shrinking vertically */
	    y -= z >> 7;
	    z += y >> 7;
	    if (y < 0) {
		y = 0;
		z = ymax;
		ts = -ts;
		ta = -ta;
		dts = -dts;
		dta = -dta;
		phase = 3;
	    }
	    break;
	case 3: 	      /* rectangle growing vertically */
	    y += z >> 7;
	    z -= y >> 7;
	    if (z < 0) {
		y = ymax;
		z = 0;
		phase = 0;
	    }
	    break;
	}

	/* Don't start drawing until scheduled page flip is done. */
	while (page_busy())
	    ;
    if (kbhit())
	break;
    }
    getch();
    clear_page(BLACK);
}

