/*----------------------------------------------------------------------*/
/*									*/
/*	Copyright (c) 1992	ELZET 80 Mikrocomputer			*/
/*				All Rights Reserved			*/
/*									*/
/*----------------------------------------------------------------------*/
/*   GDP340 - Graphics Manager						*/
/*----------------------------------------------------------------------*/
/*   Demonstration Program						*/
/*									*/
/*   Use the draw_point, pen_point and patnpen_point functions		*/
/*   to cover the screen with random points.				*/
/*									*/
/*   Notes:								*/
/*----------------------------------------------------------------------*/
/*   Revision history:							*/
/*   08.02.1993	Original Version Written		Reinhard Niebur	*/
/*----------------------------------------------------------------------*/

#include "gmtest.h"

CONFIG cfg;
FONTINFO fontinfo;
short hres,vres,x,y;
short patn,penw,penh;
char s[129];

#define NCOLORS 16

static PALET palet[NCOLORS];

/* Print error message and exit. */
void error(char *str)
{
    printf("[ Error: ");
    if (!*str)
        printf("Unexpected EOF");
    else
        printf("%s",str);
    printf(" ]\n");
    exit(1);
}

void main(void)
{
short j, k, n;

    if (!gm_init())
        error("No Response from GDP");

    /* Initialize graphics environment. */
    if (!set_config(MODE, 1))
	error("Invalid graphics mode number");
    clear_screen(-1);
    get_config(&cfg);
    hres = cfg.mode.disp_hres;	 /* horizontal resolution */
    vres = cfg.mode.disp_vres;	 /* vertical resolution */

    for (j = 0; j < 5000; j++)
    {
	x = rand() % hres;
	y = rand() % vres;
	set_colors((long)(rand() & 15),(long)(rand() & 15));
	switch(rand() & 3)
	{
	    case 0: draw_point(x,y); break;
	    case 1: set_pensize(8,8);
		    pen_point(x,y); break;
	    case 2: set_hard_patn(rand() & 31);
		    set_pensize(16,16);
		    patnpen_point(x & 0xfff0, y & 0xfff0); break;
	}
    }

    getch();
    for (n = 0; n < 16; n++)
	set_palet_entry((long)n,
		  (uchar)(16*n),(uchar)(16*n),(uchar)(16*n),(uchar)(16*n));
    delay(60);

    for (k = 0; k < NCOLORS; k++)
    {
	delay(5);
	get_palet(NCOLORS, palet);
	for(j = 0; j < NCOLORS; j++)
	{
	    if(palet[j].r > 16) palet[j].r -= 16;
	    else palet[j].r = 0;
	    if(palet[j].g > 16) palet[j].g -= 16;
	    else palet[j].g = 0;
	    if(palet[j].b > 16) palet[j].b -= 16;
	    else palet[j].b = 0;
	    if(palet[j].i > 16) palet[j].i -= 16;
	    else palet[j].i = 0;
	}
	set_palet(NCOLORS, 0, palet);
    }
    clear_page(BLACK);
}

