/*----------------------------------------------------------------------*/
/*									*/
/*	Copyright (c) 1992	ELZET 80 Mikrocomputer			*/
/*				All Rights Reserved			*/
/*									*/
/*----------------------------------------------------------------------*/
/*   GDP340 - Graphics Manager						*/
/*----------------------------------------------------------------------*/
/*   Demonstration Program						*/
/*									*/
/*   Use draw_oval, pen_ovalarc and patpen_ovalarc to draw an ellipse.	*/
/*   Use get_pixel, set_pixel and move_pixel to rotate a text line.	*/
/*   Demonstrate plane mask usage.					*/
/*   Do various tests to confirm functions:				*/
/*   get_env, get_pmask, set_pmask, lmo, peek_breg, poke_breg,		*/
/*   field_extract, field_insert, transp_on, transp_off,		*/
/*   get_ppop, set_ppop, get_windowing, set_windowing.			*/
/*									*/
/*   Notes:								*/
/*----------------------------------------------------------------------*/
/*   Revision history:							*/
/*   08.02.1993	Original Version Written		Reinhard Niebur	*/
/*----------------------------------------------------------------------*/

#include "gmtest.h"

CONFIG cfg;
FONTINFO fontinfo;
short hres,vres,x,y;
short patn,penw,penh;
char s[129];

#define CONTROL (PTR)(0xc00000b0L)

static ENVIRONMENT env;

/* Print error message and exit. */
void error(char *str)
{
    printf("[ Error: ");
    if (!*str)
        printf("Unexpected EOF");
    else
        printf("%s",str);
    printf(" ]\n");
    exit(1);
}

void main(void)
{
short w, h;
short xs, ys, xd, yd;
long pmask, val;
char *str;

    if (!gm_init())
        error("No Response from GDP");

    /* Initialize graphics environment. */
    if (!set_config(MODE, 1))
	error("Invalid graphics mode number");
    clear_screen(LIGHT_GRAY);
    get_config(&cfg);
    hres = cfg.mode.disp_hres;	 /* horizontal resolution */
    vres = cfg.mode.disp_vres;	 /* vertical resolution */
    get_fontinfo(0, &fontinfo);

    set_draw_origin(100,200);
    get_env(&env);
    if (env.xyorigin != 0x00c80064L)
	error("Cannot read environment");

    set_fcolor(BLACK);
    if (peek_breg(9) != 0l)
	error("Cannot read register B9");
    poke_breg(9, 0xffffffffl);
    if (peek_breg(9) != 0xffffffffl)
	error("Cannot write register B9");

    transp_on();
    if (get_transp() != 1)
	error("get_transp()");
    if (field_extract(CONTROL+5,1) != 1)
	error("Cannot field_extract(CONTROL+5,1)");
    field_insert(CONTROL+5,1,0);
    if (field_extract(CONTROL+5,1) != 0)
	error("Cannot field_insert(CONTROL+5,1,0)");
    transp_off();
    if (get_transp() != 0)
	error("transp_off()");

    field_insert(CONTROL+10,5,15);
    if (get_ppop() != 15)
	error("get_ppop()");
    set_ppop(0);
    if (get_ppop() != 0)
	error("set_ppop()");

    field_insert(CONTROL+6,2,0);
    if (get_windowing() != 0)
	error("get_windowing()");
    set_windowing(3);
    if (get_windowing() != 3)
	error("set_windowing()");

    set_draw_origin(0,0);
    w = hres/2-40;
    h = vres/2-40;
    x = 40;
    y = 40;
    set_pensize(9, 9);
    set_fcolor(LIGHT_GREEN);
    pen_ovalarc(w, h, x, y, 0, 330);
    set_fcolor(RED);
    draw_oval(w, h, x, y);

    set_draw_origin(hres/2-40, vres/3);
    set_hard_patn(15);
    set_pensize(16,16);
    set_colors(LIGHT_GREEN, BLUE);
    patnpen_ovalarc(w, h, x, y, 0, 330);
    set_fcolor(RED);
    draw_oval(w, h, x, y);
    
    set_colors(DARK_GRAY,LIGHT_GRAY);
    str = "Rotate text by 180 degrees using get/put_pixel";
    w = text_width(str);
    h = fontinfo.charhigh;
    set_draw_origin(hres/4, vres-3*h);
    xs = 0;
    ys = 0;
    text_out(xs, ys, str);
    for(yd = 2*h+1; ys <= h; ys++, yd--)
	for(xs = 0, xd = w-1; xs <= w; xs++, xd--)
	{
	    val = get_pixel(xs, ys);
	    put_pixel(val, xd, yd);
	}

    set_draw_origin(10, 10);
    str = "Rotate text by 90 degrees using move_pixel";
    w = text_width(str);
    h = fontinfo.charhigh;
    xs = h;
    ys = 0;
    text_out(xs, ys, str);
    for(xd = yd = h; ys < h; ys++, xd = h-ys, yd = h)
	for(xs = h; xs < w+h; xs++, yd++)
	{
	    move_pixel(xs, ys, xd, yd);
	}

    set_draw_origin(hres/2+20, 40);
    set_colors(WHITE, BLACK);
    x = y = 0;
    for(pmask = 1; pmask != 1<<4; pmask <<= 1)
    {
	set_pmask(pmask);
	sprintf(s,"All planes  enabled except %d (%04lx)",
		lmo(pmask), get_pmask());
	text_out(x, y, s);
	y += fontinfo.charhigh;
	set_pmask(~pmask);
	sprintf(s,"All planes disabled except %d (%04lx)",
		lmo(pmask), get_pmask());
	text_out(x, y, s);
	y += fontinfo.charhigh;
    }
    set_pmask(0);
    sprintf(s,"All planes  enabled          (%04lx)", get_pmask());
    text_out(x, y, s);
    getch();
    clear_page(BLACK);
}

