/*----------------------------------------------------------------------*/
/*									*/
/*	Copyright (c) 1992	ELZET 80 Mikrocomputer			*/
/*				All Rights Reserved			*/
/*									*/
/*----------------------------------------------------------------------*/
/*   GDP340 - Graphics Manager						*/
/*----------------------------------------------------------------------*/
/*   Demonstration Program						*/
/*									*/
/*   Text screen demo with build in fonts.				*/
/*   Show build in patterns.						*/
/*									*/
/*   Notes:								*/
/*----------------------------------------------------------------------*/
/*   Revision history:							*/
/*   12.02.1993	Original Version Written		Reinhard Niebur	*/
/*----------------------------------------------------------------------*/

#include "gmtest.h"

CONFIG cfg;
FONTINFO fontinfo;
short hres,vres,x,y;
short patn,penw,penh;
char s[129];

/* Print error message and exit. */
void error(char *str)
{
    printf("[ Error: ");
    if (!*str)
        printf("Unexpected EOF");
    else
        printf("%s",str);
    printf(" ]\n");
    exit(1);
}

void show_fonts(void)
{
short i;
    set_colors(WHITE,BLUE);
    clear_page(-1);
    x = y = 0;
    for(i=1; i <= N_HARD_FONTS; i++, y += fontinfo.charhigh)
    {
	if(select_font(i))
	{
	    if(get_fontinfo(-1,&fontinfo))
	    {
		sprintf(s,"%1d",i);
		set_colors(BLUE,WHITE);
		x = text_out(0,y,s);
		if(fontinfo.charwide)
		    sprintf(s,":%s%3d*%d",
			fontinfo.facename,fontinfo.charwide,fontinfo.charhigh);
		else
		    sprintf(s,":%s %d*%d",
			fontinfo.facename,fontinfo.avgwide,fontinfo.charhigh);
		set_colors(WHITE,BLUE);
		x = text_out(x,y,s);
	    }
	    else
	    {
		select_font(0);
		x = text_out(2,y,"ERROR getting fontinfo");
	    }
	}
	else
	{
	    select_font(0);
	    x = text_out(2,y,"ERROR selecting font");
	}
    }
    select_font(ARROWS31);
    x = text_out(0,y,"\125");
    select_font(CORPUS49);
    x = text_out(x,y,"Select (ESC to abort)");
}

void show_font(short nr)
{
short i;
char key[2];
    key[1] = '\0';
    select_font(nr);
    get_fontinfo(nr,&fontinfo);
    clear_page(-1);
    x = y = 0;
    set_fcolor(YELLOW);
    for (i=0; i<=16; i++)
    {
	draw_line(0,i*fontinfo.charhigh + i,
			16*(fontinfo.avgwide+1),i*fontinfo.charhigh + i);
	draw_line(i*fontinfo.avgwide + i,0,
			i*fontinfo.avgwide + i,16*(fontinfo.charhigh+1));
    }
    set_fcolor(WHITE);
    for (i=0; i<256; i++)
    {
	x = (i & 15) * fontinfo.avgwide + (i & 15) + 1;
	y = (i / 16) * fontinfo.charhigh + i / 16 + 1;
	set_text_xy(x,y);
	key[0] = i;
	text_out(x,y,key);
    }
    i = getch();
    show_fonts();
}

void main(void)
{
short i,x1;

    if (!gm_init())
        error("No Response from GDP");

    set_config(MODE,!0);
    get_config(&cfg);
    hres = cfg.mode.disp_hres;
    vres = cfg.mode.disp_vres;

    penw = penh = 1;
    patn = 15;
    set_colors(WHITE,BLUE);
    clear_screen(BLUE);
    for(i = 0; i < N_HARD_FONTS; i++)
	install_hard_font(i);
    select_font(SYS);
    set_textattr("%1f",0,0);	/* Set Text Attr: Fill Gaps */
    get_fontinfo(-1,&fontinfo);	/* Get info of currend font */

    y = 8;
    strcpy(s,"Draw a border around Text with Draw Line");
    x = hres/2 - text_width(s)/2;	/* Center text */
    x1 = text_out(x,y,s);
    set_fcolor(RED);
    draw_line(x-2,y-2,x1+2,y-2);
    draw_line(x-2,y-2,x-2,y+fontinfo.charhigh+2);
    draw_line(x-2,y+fontinfo.charhigh+2,x1+2,y+fontinfo.charhigh+2);
    draw_line(x1+2,y-2,x1+2,y+fontinfo.charhigh+2);

    set_fcolor(GREEN);
    y += fontinfo.charhigh + 10;
    strcpy(s,"The same procedure in green with Draw_rect");
    x = hres/2 - text_width(s)/2;
    text_out(x,y,s);
    set_fcolor(LIGHT_GREEN);
    draw_rect(2+text_width(s)+2,2+fontinfo.charhigh+2,x-2,y-2);

    set_fcolor(LIGHT_MAGENTA);
    y += fontinfo.charhigh + 10;
    strcpy(s,"Homemade styled lines are possible too with Styled_Line");
    x = hres/2 - text_width(s)/2;
    x1 = text_out(x,y,s);
    set_colors(LIGHT_CYAN,RED);
    styled_line(x-2,y-2,x1+2,y-2,0x18ff18ffL,0);
    styled_line(x-2,y-2,x-2,y+fontinfo.charhigh+2,0,2);
    styled_line(x-2,y+fontinfo.charhigh+2,
    		x1+2,y+fontinfo.charhigh+2,0x0f0f0f0fL,1);
    styled_line(x1+2,y-2,x1+2,y+fontinfo.charhigh+2,0,3);

    set_colors(LIGHT_GRAY,DARK_GRAY);
    y += fontinfo.charhigh + 14;
    strcpy(s,"... or lines with various width and high with Pen_Line");
    x = hres/2 - text_width(s)/2;
    x1 = text_out(x,y,s);
    set_colors(YELLOW,BLUE);
    penw = penh = 4;
    set_pensize(penw,penh);
    pen_line(x-penw-2,y-penh-2,x1+2,y-penh-2);
    pen_line(x-penw-2,y-penh-2,x-penw-2,y+fontinfo.charhigh+2);
    penw = penh = 6;
    set_pensize(penw,penh);
    pen_line(x-penw,y+fontinfo.charhigh+2,x1+2,y+fontinfo.charhigh+2);
    pen_line(x1+2,y-penh,x1+2,y+fontinfo.charhigh+2);
    set_fcolor(DARK_GRAY);
    penw = penh = 4;
    set_pensize(penw,penh);
    pen_line(x-penw+4,y+fontinfo.charhigh+8,x1+6,y+fontinfo.charhigh+8);
    pen_line(x1+8,y-penh,x1+8,y+fontinfo.charhigh+8);

    set_colors(LIGHT_MAGENTA,GREEN);
    y += fontinfo.charhigh + 28;
    strcpy(s,"... or pen lines filled with various patterns with PatnPen_Line");
    x = hres/2 - text_width(s)/2;
    x1 = text_out(x,y,s);
    set_colors(LIGHT_RED,CYAN);
    set_hard_patn(5);
    penw = penh = 10;
    set_pensize(penw,penh);
    patnpen_line(x-penw-2,y-penh-2,x1+2,y-penh-2);
    patnpen_line(x-penw-2,y-penh-2,x-penw-2,y+fontinfo.charhigh+2);
    penw = penh = 16;
    set_pensize(penw,penh);
    patnpen_line(x-penw+4,y+fontinfo.charhigh+2,x1+2,y+fontinfo.charhigh+2);
    patnpen_line(x1+2,y-penh+4,x1+2,y+fontinfo.charhigh+2);

    set_colors(LIGHT_CYAN,LIGHT_CYAN);
    y += fontinfo.charhigh + 26;
    strcpy(s,"Windows with different background colors are easy to made with Fill_Rect");
    x1 = text_width(s);
    x = hres/2 - x1/2;
    fill_rect(x1+32,fontinfo.charhigh+8,x-16,y-4);
    set_fcolor(MAGENTA);
    text_out(x,y,s);

    set_fcolor(LIGHT_CYAN);
    y += fontinfo.charhigh + 10;
    strcpy(s,"The same but switched to an different background color when drawing text");
    x1 = text_width(s);
    x = hres/2 - x1/2;
    fill_rect(x1+32,fontinfo.charhigh+8,x-16,y-4);
    set_colors(MAGENTA,LIGHT_GREEN);
    text_out(x,y,s);

    strcpy(s,"PatnFill_Rect: The same with an wonderful background pattern! Isn't it!?");
    x1 = text_width(s);
    x = hres/2 - x1/2;
    y += 34 + fontinfo.charhigh;
    set_text_xy(x, y);
    set_colors(DARK_GRAY,LIGHT_GRAY);
    set_hard_patn(14);
    patnfill_rect(x1+32,64,x-16,y-24);
    set_colors(LIGHT_GRAY,DARK_GRAY);
    text_out(x,y,s);

    set_fcolor(MAGENTA);
    y += 12 + 48;
    strcpy(s,"To draw wide borders we prefer function Frame_Rect");
    x1 = text_width(s);
    x = hres/2 - x1/2;
    frame_rect(x1+24,fontinfo.charhigh+24,x-4-8,y-4-8,8,8);
    set_colors(YELLOW,BROWN);
    text_out(x,y,s);

    y += 24 + 14 + fontinfo.charhigh;
    y = (y + 15) & 0xfff0;
    strcpy(s,"AlL WhaT LookS LikE ArT IS ArT!");
    x1 = text_width(s);
    x = hres/2 - x1/2;
    set_colors(BROWN,LIGHT_BLUE);
    set_hard_patn(2);
    patnframe_rect(hres/2,32 + 24 + 2*fontinfo.charhigh,hres/4,y-24,16,16);
    set_colors(MAGENTA,LIGHT_GREEN);
    strcpy(s,"PatnFrame_Rect:");
    x = hres/2 - text_width(s)/2;
    text_out(x,y,s);
    strcpy(s,"AlL WhaT LookS LikE ArT IS ArT!");
    x = hres/2 - text_width(s)/2;
    y += fontinfo.charhigh + 8;
    text_out(x,y,s);

    i = getch();

    show_fonts();
    while ((i = getch()) != 0x1b)
    {
	switch (i) {
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9': show_font(i-'0'); break;
	}
    }

/* Show all patterns with Patn Pen Line */
    set_colors(WHITE,BLUE);
    clear_screen(-1);
    select_font(SYS);
    x = 0;
    y = 16;
    set_pensize(32,32);
    patn = 0;
    for (i=0; i<32; i++)
    {
	set_colors(WHITE,BLUE);
	sprintf(s,"Pattern %2d",i);
	text_out((i&3)*128+44,y+32,s);
	set_colors(i,15-i);
	set_hard_patn(patn++);
	patnpen_line((i&3)*128+64,y,(i&3)*128+64,y);
	if((i&3) == 3)
	    y += 48;
    }
    set_colors(WHITE,BLUE);
    x = 16;
    y += 16;
    draw_rect(32*16+1,33,x-1,y-1);
    for (i=0; i<16; i++, x+=32)
    {
	set_fcolor(i);
	fill_rect(32,32,x,y);
	set_fcolor(WHITE);
	sprintf(s,"%d",i);
	x1 = x + 16 - text_width(s)/2;
	text_out(x1,y+34,s);
    }
    i = getch();
}
