/*----------------------------------------------------------------------*/
/*									*/
/*	Copyright (c) 1992	ELZET 80 Mikrocomputer			*/
/*				All Rights Reserved			*/
/*									*/
/*----------------------------------------------------------------------*/
/*   GDP340 - Graphics Manager						*/
/*----------------------------------------------------------------------*/
/*   Demonstration Program						*/
/*									*/
/*   Simple window demo.						*/
/*									*/
/*   Notes: very simple, proper results only with block fonts		*/
/*----------------------------------------------------------------------*/
/*   Revision history:							*/
/*   13.02.1993	Original Version Written		Reinhard Niebur	*/
/*----------------------------------------------------------------------*/

#include "gmtest.h"

CONFIG cfg;
FONTINFO fontinfo;
short hres,vres,x,y;
short patn,penw,penh;
char s[129];

/* Print error message and exit. */
void error(char *str)
{
    printf("[ Error: ");
    if (!*str)
        printf("Unexpected EOF");
    else
        printf("%s",str);
    printf(" ]\n");
    exit(1);
}

#define	W_CLOSE		0
#define	W_OPEN		1
#define	WRAPOFF		0
#define	WRAPON		1

typedef struct WINDOW {
	short	x, y;
	ushort	w, h;
	short	xa, ya;
	ushort	wa, ha;
	char	*name;
	short	border;
	short	cursx, cursy;
	struct	WINDOW *prev;
	PTR	bgnd;
	ushort	open;
} Window;

Window *w_top;

Window w1 = {
    10,10,400,200,0,0,0,0,"Window 1",1,0,0,0,0,W_CLOSE };
Window w2 = {
    30,30,400,200,0,0,0,0,"Window 2",0,0,0,0,0,W_CLOSE };
Window w3 = {
    50,50,400,200,0,0,0,0,"Window 3",4,0,0,0,0,W_CLOSE };
Window w4 = {
    70,70,400,140,0,0,0,0,(char *)0,4,0,0,0,0,W_CLOSE };
Window w5 = {
    100,100,497,178,0,0,0,0,"Window 5",4,0,0,0,0,W_CLOSE };
Window w6 = {
    1,110,638,345,0,0,0,0,"Type (ESC to exit)",4,0,0,0,0,W_CLOSE };

char *w1_text = "\
This is window number 1.\n\
It has an 1 pixel wide border and a title.\n\
Its position is at 10,10,\n\
its width and height is 400,200.";

char *w2_text = "\
This is window number 2.\n\
It has no border but also a title.\n\
Its position is at 30,30,\n\
its width and height is 400,200."; 

char *w3_text = "\
This is window number 3.\n\
It has an 4 pixel wide border and a title.\n\
Its position is at 50,50,\n\
its width and height is 400,200.";

char *w4_text = "\
This is window number 4.\n\
It has an 4 pixel wide border and no title.\n\
Its position is at 70,70,\n\
its width and height is 400,140.";

char *w5_text = "This is scrolling in a window.";

short w_close(void);

void w_init(void)
{
    w_top = (Window *)0;
}

void w_exit(void)
{
    while(w_top != (Window *)0)
	w_close();
}

short w_cursor(short x, short y)
{
    if (w_top->open == W_CLOSE || w_top == (Window *)0) return(FALSE);
    w_top->cursx = x;
    w_top->cursy = y;
    return(TRUE);
}

short w_clear_screen(long color)
{
ulong f, b;
    if (w_top->open == W_CLOSE || w_top == (Window *)0) return(FALSE);
    get_colors(&f, &b);
    if(color == -1)
	set_fcolor(b);
    else
	set_fcolor(color);
    fill_rect(w_top->wa, w_top->ha, 0, 0);
    set_colors(f, b);
    return(TRUE);
}

short w_text(char *s, short wrap)
{
Window *w;
short i, fh, fw;
ulong f, b;
char key[2];
    w = w_top;
    if (w->open == W_CLOSE || w == (Window *)0 || s == 0) return(FALSE);
    fh = fontinfo.charhigh;
    fw = fontinfo.charwide;
    get_colors(&f, &b);
    key[1] = '\0';
    i = 0;
    while((key[0] = s[i++]) != 0)
    {
	if ( ((w->cursx > w->wa-fw) && wrap)
	  || (*key == '\n'))
	{
	    w->cursx = 0;
	    w->cursy += fh;
	}
	if (w->cursy > w->ha-fh)
	{
	    bitblt(w->wa, w->ha-fh, 0, fh, 0, 0);
	    w->cursy -= fh;
	    set_fcolor(b);
	    fill_rect(w->wa, fh, 0, w->cursy);
	    set_fcolor(f);
	}
	if (*key != '\n')
	    w->cursx = text_out(w->cursx, w->cursy, key);
    }
    return(TRUE);
}

short w_read(char *s, short len)
{
Window *w = w_top;
char c,key[2];
short i, fw;
    if (w->open == W_CLOSE || w == (Window *)0 || len == 0) return(FALSE);
    fw = fontinfo.charwide;
    key[1] = '\0';
    w_text("_", WRAPOFF);
    i = 0;
    while((c=getch()) != '\r')
    {
	if (c == '\b' && i > 0)
	{
	    w->cursx -= 2*fw;
	    w_text("_ ", WRAPOFF);
	    i--;
	    w->cursx -= fw;
	}
	else
	    if (i < len && c != '\b')
	    {
		s[i++] = c;
		w->cursx -= fw;
		key[0] = c;
		w_text(key, WRAPOFF);
		w_text("_", WRAPOFF);
	    }
    }
    s[i] = '\0';
    w->cursx -= fw;
    w_text(" ", WRAPOFF);
    return(TRUE);
}

short w_open(Window *w)
{
short dx = 0;
short dy = 0;
ulong siz;
short old_id;
FONTINFO fnt;
    if (w->open == W_OPEN)
	return(FALSE);
    set_draw_origin(0, 0);
    set_clip_rect(hres-1, vres-1, 0, 0);
    siz = encode_rect(w->w, w->h, w->x, w->y, (PTR)0, 0, 0);
    if (siz)
    {
 	if((w->bgnd = gsp_malloc(siz)) != 0)
	{
	    encode_rect(w->w, w->h, w->x, w->y, w->bgnd, siz, 0);
	    set_fcolor(BLACK);
	    frame_rect(w->w, w->h, w->x, w->y, w->border, w->border);
	    dx += w->border;
	    dy += w->border;
	    if (w->name)
	    {
		old_id = fontinfo.id;
		select_font(SYS16);
		get_fontinfo(-1,&fnt);
		set_fcolor(LIGHT_BLUE);
		fill_rect(w->w-2*w->border, fnt.charhigh,
			w->x+w->border, w->y+w->border);
		set_colors(WHITE,LIGHT_BLUE);
		text_out(w->x + (w->w-w->border)/2 - text_width(w->name)/2,
			w->y+w->border, w->name);
		dy += fnt.charhigh;
		select_font(old_id);
	    }
	    set_colors(BLACK,WHITE);
	    w->xa = w->x + dx;
	    w->ya = w->y + dy; 
	    w->wa = w->w - w->border - dx;
	    w->ha = w->h - w->border - dy; 
	    set_draw_origin(w->xa, w->ya);
	    set_clip_rect(w->wa, w->ha, 0, 0);
	    w->prev = w_top;
	    w_top = w;
	    w->open = W_OPEN;
	    w_clear_screen(WHITE);
	    return(TRUE);
	}
    }
    set_draw_origin(w_top->x, w_top->y);
    set_clip_rect(w_top->w, w_top->h, 0, 0);
    return(FALSE);
}

short w_close(void)
{
Window *w;
    w = w_top;
    if (w->open == W_CLOSE || w == (Window *)0) return(FALSE);
    set_draw_origin(0, 0);
    set_clip_rect(hres-1, vres-1, 0, 0);
    decode_rect(w->x, w->y, w->bgnd);
    gsp_free(w->bgnd);
    w->open = W_CLOSE;
    w_top = w->prev;
    w->prev = 0;
    set_draw_origin(w_top->xa, w_top->ya);
    set_clip_rect(w_top->wa, w_top->ha, 0, 0);
    return(TRUE);
}

void main(void)
{
short i;
char key[2];

    if (!gm_init())
        error("No Response from GDP");

    set_config(MODE,!0);
    get_config(&cfg);
    hres = cfg.mode.disp_hres;
    vres = cfg.mode.disp_vres;

    set_colors(BLACK,WHITE);
    clear_screen(-1);
    for(i = 0; i < N_HARD_FONTS; i++)
	install_hard_font(i);
    select_font(SYS);
    set_textattr("%1f",0,0);	/* Set Text Attr: Fill Gaps */
    get_fontinfo(-1,&fontinfo);	/* Get info of currend font */

    w_init();
    w_open(&w1);
    w_text(w1_text, WRAPON);
    getch();
    w_open(&w2);
    w_text(w2_text, WRAPON);
    getch();
    w_open(&w3);
    w_text(w3_text, WRAPON);
    getch();
    w_open(&w4);
    w_text(w4_text, WRAPON);
    getch();
    w_open(&w5);
    while(kbhit() == 0)
	w_text(w5_text, WRAPON);
    getch();
    w_open(&w6);
    key[1] = 0;
    while((key[0] = getch()) != 0x1b)
	w_text(key, WRAPOFF);
    while(w_close())
	getch();
    w_exit();
    clear_page(BLACK);
}
